#!/usr/bin/env python
# -*- coding:utf-8 -*-

from bson.objectid import ObjectId
from lcyframe import utils
from base import BaseSchema

class AdminSchema(BaseSchema):
    """
    admin ******
    """
    collection = "admin"

    def __init__(self):
        self._id = ObjectId()
        self.uid = 0    # ******uid
        self.user_name = ''
        self.nick_name = ""  # ******
        self.pass_word = ""  # ****** MD5(******+salt)
        self.salt = utils.gen_salt()  # ******
        self.mobile = ''  # ******
        self.email = ""  # ******
        self.sex = 1  # ****** 0:******  1:******
        self.gid = 1024  # ****** 1024 ****** 2 ******
        self.state = 1



# coding=utf-8
import datetime
import logging
import redis
import pyssdb
import nsq
import threading
from lcyframe.libs.emchat import EMChatAsync
from lcyframe.libs.hot_queue import HotQueue
from lcyframe.libs.mob_sms import MobSMS
from lcyframe.libs.smtp import SmtpServer
from lcyframe.libs.mqtt_route import ConnectMqtt
try:
    from AsyncQiniu.sevencow import Cow as Qiniu
except:
    pass

try:
    from pymongo import MongoClient, ReadPreference
except:
    print("not found model pymongo")

# try:
#     from database.base_mysqldb import MysqlDb
# except:
#     print("not found model MysqlDB")
#

class MongoCon(object):
    """mongodb数据库连接单例类"""
    _database = {}

    @classmethod
    def get_connection(cls, **kwargs):
        """返回全局的数据库连接
        return mongodb connection
        """
        host = kwargs.pop("host", "localhost")
        port = kwargs.pop("port", 27017)
        document_class = kwargs.pop("document_class", dict)
        tz_aware = kwargs.pop("tz_aware", None)
        connect = kwargs.pop("connect", None)
        type_registry = kwargs.pop("type_registry", None)

        options = kwargs

        is_repliset = kwargs.pop("model", "single").lower() in ["repliset", "replica"]

        if is_repliset:
            """
            读取模式
            primary	默认模式。只从当前复制集主节点读取
            primaryPreferred	大部分情况都从，主节点读取数据。当主节点不可用，从从节点读取
            secondary	只从从节点读取数据。仅当从节点不可用，从主节点读取
            secondaryPreferred	大部分情况从 从节点读取，当从节点都不可用，从主节点读取
            nearest	从延时最小的节点读取，不管主节点还是从节点
            """
            mp = {
                "primary": ReadPreference.PRIMARY,
                "primaryPreferred": ReadPreference.PRIMARY_PREFERRED,
                "secondary": ReadPreference.SECONDARY,
                "secondaryPreferred": ReadPreference.SECONDARY_PREFERRED,
                "nearest": ReadPreference.PRIMARY,
            }
            options["read_preference"] = mp[kwargs.get("read_preference", "primary")]
        else:
            kwargs.pop("model", "")
            kwargs.pop("replicaSet", "")
            kwargs.pop("read_preference", "")

        options.update(kwargs)

        return MongoClient(
                        host=host,
                        port=port,
                        document_class=document_class,
                        tz_aware=tz_aware,
                        connect=connect,
                        type_registry=type_registry,
                        **options)

    @classmethod
    def get_database(cls, **kwargs):
        """返回当前数据库
        return mongodb database
        """
        if type(kwargs.get("host")) is list:
            conn_key = "/".join(kwargs["host"])
        else:
            conn_key = "%s.%s" % (kwargs.get("host", "localhost"), str(kwargs.get("port", "27017")))

        database = kwargs.pop("database", "test")
        conn_key = conn_key + ":" + database

        if conn_key not in cls._database:
            cls._database[conn_key] = cls.get_connection(**kwargs)[database]
        return cls._database[conn_key]


class PyMysqlCon(object):
    _mysql = None

    @classmethod
    def get_connection(cls, **kwargs):
        mode = int(kwargs.get("mode", 3))    # 连接模式 1 PyMysqlCon，2 PyMysqlThreadCon， 3 PyMysqlPooledDB， 4 PyMysqlPersistentDB
        from lcyframe.libs.database.base_pymysql import PyMysqlPooledDB, PyMysqlPersistentDB
        mp = {
            # 1: PyMysqlCon,
            # 2: PyMysqlThreadCon,
            3: PyMysqlPooledDB,
            4: PyMysqlPersistentDB
        }
        if cls._mysql is None:
            cls._mysql = mp[mode](**kwargs)
        return cls._mysql


class RedisCon(object):
    """redis连接单例类"""
    _redis = None

    @classmethod
    def get_connection(cls, **kwargs):
        if cls._redis is None:
            cls._redis = redis.Redis(**kwargs)
        return cls._redis


class SSDBCon(object):
    """SSDB连接单例类"""
    _ssdb = None

    @classmethod
    def get_connection(cls, **kwargs):
        if cls._ssdb is None:
            cls._ssdb = pyssdb.Client(kwargs.get("host", "127.0.0.1"), int(kwargs.get("port", 8888)), max_connections=1048576)
        return cls._ssdb


class MQCon(object):
    """message queue 连接单例类"""
    _mq = None

    @classmethod
    def get_connection(cls, **kwargs):
        if cls._mq is None:
            import json
            cls._mq = HotQueue(name=kwargs.get("name", "mq"),
                               serializer=json,     # pickle has a decode error with py3
                               host=kwargs.get("host", "127.0.0.1"),
                               port=int(kwargs.get("port", 6379)),
                               db=kwargs.get("db", 0))
        return cls._mq


class NsqCon(object):
    """message nsq 连接单例类"""
    _nsq = None

    @classmethod
    def get_connection(cls, **kwargs):
        if cls._nsq is None:
            cls._nsq = nsq.Writer(kwargs.get("nsqd_tcp_addresses", "127.0.0.1:4150"))
        return cls._nsq


class MqttCon(object):
    """message mqtt 连接单例类"""
    _mqtt = None

    @classmethod
    def get_connection(cls, **kwargs):
        if cls._mqtt is None:
            # 代理模式，使用publish.single()提交数据
            # from mqtt_route import ReadMqtt
            # cls._mqtt = ReadMqtt(**kwargs)

            # 生产者长连接, loop_start()用一个独立线程保持连接，代替每次loop()；消费客户端使用loop_forever()轮训；二者互斥，单线程内互踢链接。
            cls._mqtt = ConnectMqtt(**kwargs).mqttc
        return cls._mqtt


class EmchatCon(object):
    """message queue 连接单例类"""
    _emchat = None

    @classmethod
    def get_connection(cls, **kwargs):
        if cls._emchat is None:
            cls._emchat = EMChatAsync(**kwargs)
        return cls._emchat


class SMSCon(object):
    """message queue 连接单例类"""
    _sms = None

    @classmethod
    def get_connection(cls, **kwargs):
        if cls._sms is None:
            cls._sms = MobSMS(**kwargs)
        return cls._sms

class SmtpCon(object):
    """email queue 连接单例类"""
    _smtp = None

    @classmethod
    def get_connection(cls, **kwargs):
        if cls._smtp is None:
            cls._smtp = SmtpServer(**kwargs)
        return cls._smtp


class QinNiuCon(object):
    """message queue 连接单例类"""
    _qiniu = None

    @classmethod
    def get_connection(cls, **kwargs):
        if cls._qiniu is None:
            cls._qiniu = Qiniu(**kwargs)
        return cls._qiniu


class CeleryCon(object):
    _app = None
    _instance_lock = threading.Lock()

    @classmethod
    def get_connection(cls, **kwargs):
        if cls._app is None:
            from .celery_route import MyCelery
            cls._app = MyCelery(kwargs.get("project", "lcyframe-celery"))
            # app.config_from_object("celeryconfig")
            # os.environ['CELERY_CONFIG_MODULE'] = 'celeryconfig'
            cls._app.config_from_envvar('CELERY_CONFIG_MODULE')
        return cls._app



    # @classmethod
    # def get_connection(cls, **kwargs):
    #     if not hasattr(CeleryCon, "_app"):
    #         with CeleryCon._instance_lock:
    #             if not hasattr(CeleryCon, "_app"):
    #                 from celery_route import MyCelery
    #                 CeleryCon._app = MyCelery(kwargs.get("project", "lcyframe-celery"))
    #                 # app.config_from_object("celeryconfig")
    #                 # os.environ['CELERY_CONFIG_MODULE'] = 'celeryconfig'
    #                 CeleryCon._app.config_from_envvar('CELERY_CONFIG_MODULE')
    #     return CeleryCon._app

class BeanstalkCon(object):
    _conn = None

    @classmethod
    def get_connection(cls, **kwargs):
        if cls._conn is None:
            from .beanstalk import BeanStalk
            cls._conn = BeanStalk(host=kwargs["host"], port=kwargs["port"], watch=kwargs.get("use", "default"))

        return cls._conn

class GrayLogCon(object):
    """
    收集异常日志、定向发送日志
    """
    _instance_lock = threading.Lock()

    def __init__(self, **kwargs):
        from gelfclient import UdpClient
        try:
            self.graylog = UdpClient(kwargs.get("host", '127.0.0.1'), kwargs.get("port", 12201))
        except Exception as e:
            self.graylog = GrayLogCon()

    def __new__(cls, **kwargs):
        if not hasattr(GrayLogCon, "_instance"):
            with GrayLogCon._instance_lock:
                if not hasattr(GrayLogCon, "_instance"):
                    GrayLogCon._instance = object.__new__(cls)
        return GrayLogCon._instance

    def send(self, **data):
        """
        发送日志
        :param data:
        :return:
        """
        try:
            default = {
                "level": 1,
                "track": "",
                "host": "",
                # "timestamp": "2021-05-10 18:47:21.576 +08:00",
                "short_message": "null"
            }
            default.update(data)
            if self.graylog is None:
                logging.warning("Graylog Connection Loss.")
            else:
                self.graylog.log(default)
        except Exception as e:
            logging.warning(e)

    def from_server(self, **data):
        data["app"] = "server"
        return self.send(**data)

    def from_mongo(self, **data):
        data["app"] = "mongo"
        return self.send(**data)

    def from_mysql(self, **data):
        data["app"] = "mysql"
        return self.send(**data)

    def from_mq(self, **data):
        data["app"] = "mq"
        return self.send(**data)

class WebScoketCon(object):
    _instance = None

    @classmethod
    def get_instance(cls, **kwargs):
        if cls._instance is None:
            from ..websocket_server import WebSocketClient
            cls._instance = WebSocketClient(**kwargs)

        return cls._instance


#!/usr/bin/env python
# -*- coding:utf-8 -*-
import sys, os
root_dir = os.path.dirname(os.path.dirname(__file__))
sys.path.append(root_dir)
from test_script import *
from lcyframe.libs import cprint



def demo():
    headers = {"uid": 100000}

    params = {
        "a": 1
    }

    return admin_login_and_send(
        methed="get",
        url="/demo",
        params=params,
        headers=headers,
    )


if __name__ == "__main__":
    demo()




#!/usr/bin/env python
# -*- coding:utf-8 -*-
import os
from lcyframe import route
from lcyframe import funts
from base import BaseHandler, helper
import re

@route("/upload")
class UploadHeadHandler(BaseHandler):
    """
    上传
    """

    @helper.admin()
    def post(self):
        """

        :return:
        :rtype:
        """
        filedata = self.params.pop("file")
        info = self.helper.get_fileinfo(filedata)
        if info["format"] not in ["xlsx", "slx"]:
            raise self.api_error.FileFormatError
        if info["size"] >= 1024 * 1024 * 1:
            raise self.api_error.UploadError("大小不能超过1M")

        filename, savepath = self.helper.save_filedata(filedata,
                                                       os.path.join(self.app_config["ROOT"],
                                                                    self.app_config["data_config"]["base"],
                                                                    "project")
                                                       )

        self.write_success({"file_metadata": info})


# -*- coding: utf-8 -*-
import random, time
from celery import shared_task
from lcyframe.libs.celery_route import BaseTask
from lcyframe.libs.celery_route import BaseEvent
from celery.utils.log import get_task_logger

logger = get_task_logger(__name__)

"""
# 例子一： 延迟执行
eta=datetime.utcnow() + timedelta(seconds=10),  # 在指定的时间运行，这里实现了延迟10秒运行，2022-01-13 09:32:08.643081
func.apply_async(args=(x, ), eta=eta)

# 例子2：完成后回调
func.apply_async(args=(11111, ), link=callback.s())

# 例子3：指定队列
self.celery.Events.send_email.apply_async(
    args=[
        user,
        password
    ],
    queue="default",
    routing_key="default",
)
"""


@shared_task(ignore_result=False)  # 普通函数装饰为 celery task
def func(x):
    """
    seleep时间随机
    任务多线程并发执行，谁先执行完毕，谁先输出结果
    :param x:
    :return:
    """
    n = round(random.random(), 2)
    time.sleep(n)
    logger.info('Adding {0} + {1}'.format(x, "e"))
    return x, n

@shared_task
def callback(result):
    """
    用于回调
    先执行其他task，成功后，在将其参数传入，调用当前函数
    example: func.apply_async(args=(11111, ), link=callback.s())
    :param result:
    :return:
    """
    print(f'My result was {result}')


class DefaultEvent(BaseEvent):
    queue = "for_task_A"

    @staticmethod
    @shared_task(bind=True)
    def bind(self, x):
        """
        bind=True时，task对象self会作为第一个参数自动传入
        :param self:
        :param x:
        :return:
        """
        print(self.app)
        return x

    @staticmethod
    @shared_task(ignore_result=False, queue=queue)  # 普通函数装饰为 celery task
    def add(x):
        """
        seleep时间随机
        任务多线程并发执行，谁先执行完毕，谁先输出结果
        :use self.application.celery.DefaultEvent.add.delay(111)
        :param x:
        :return:

        """
        n = round(random.random(), 2)
        time.sleep(n)

        # 同步调用add2，等待结果返回。不建议使用。如有需要，建议使用任务链
        result = DefaultEvent().add2(x)

        # 待测试
        # s = DefaultEvent.add2.apply_async(x)

        # 异步调用add2,不等待
        AsyncResult = DefaultEvent.add2.delay(x)
        result = AsyncResult.result

        return x, n

    @shared_task(bind=True, ignore_result=False)
    def add2(self, x):      # 实例方法，第一个参数self，bind=False时，需要在调用时传入空字符串；bind=True时，调用时不需要传入，此次self为task自身
        n = round(random.random(), 2)
        time.sleep(n)
        return x, n

    @staticmethod
    @shared_task(queue="for_task_A")
    def taskA(x):
        return x, "for_task_A"

    @staticmethod
    @shared_task(queue="for_task_B")
    def taskB(x):
        return x, "for_task_B"

    @staticmethod
    @shared_task(
                bind=True,                            # True时，task对象self，会作为第一个参数自动传入，可以使用任务对象的属性。self.xxx。当设置以下参数时，必须绑定
                name="tasks.test_retry",                   # 名称，默认用函数名
                queue='for_task_A',                   # 指定队列,优先级高于配置文件中的CELERY_ROUTES
                countdown=10,                         # 10s后重试
                max_retries=3,                        # 重试次数，https://docs.celeryproject.org/en/master/userguide/tasks.html#automatic-retry-for-known-exceptions
                default_retry_delay=3,                # 默认重试的间隔时间，优先级低于countdown属性
                # autoretry_for=(ReadTimeout,),       # 添加要自动重试的异常，如果所有异常都需要重试可以写Exception，默认不写所有异常都重试。，也可以通过try..execpt 手动调用重试函数
                soft_time_limit=5,                    # 任务最大执行时间
                ignore_result=False,                  # 结果需要保存
                base=BaseTask                         # 基类
                )
    def test_retry(self, target_origin_id, to_user, txt):
        try:
            resylt = "do something"
        except BaseException as e:
            # 主动调用重试方法
            """
            retry的参数可以有：
            exc：指定抛出的异常
            throw：重试时是否通知worker是重试任务
            eta：指定重试的时间／日期
            countdown：在多久之后重试（每多少秒重试一次）
            max_retries：最大重试次数
            """
            raise self.retry(args=[target_origin_id, to_user, txt],
                             countdown=BaseTask.backoff(self.request.retries),
                             exc=e
                             )

    @staticmethod
    @shared_task(
        bind=True,  # True时，task对象self，会作为第一个参数自动传入，可以使用任务对象的属性。self.xxx。当设置以下参数时，必须绑定
        name="tasks.test_retry2",  # 名称，默认用函数名
        queue='for_task_A',  # 指定队列,优先级高于配置文件中的CELERY_ROUTES
        countdown=10,  # 10s后重试
        max_retries=3,
        # 错误重试次数，https://docs.celeryproject.org/en/master/userguide/tasks.html#automatic-retry-for-known-exceptions
        default_retry_delay=3,  # 默认重试的间隔时间，优先级低于countdown属性
        # autoretry_for=(ReadTimeout,),       # 添加要自动重试的异常，如果所有异常都需要重试可以写Exception，默认不写所有异常都重试。，也可以通过try..execpt 手动调用重试函数
        soft_time_limit=5,  # 任务最大执行时间
        ignore_result=False,  # 结果需要保存
        base=BaseTask  # 基类
    )
    def test_retry2(self, target_origin_id, to_user, txt):
        """
        自动重试,未验证
        :param self:
        :param target_origin_id:
        :param to_user:
        :param txt:
        :return:
        """
        pass
        # do something
        raise Exception()






import datetime
import logging
try:
    import pymysql
    from dbutils.pooled_db import PooledDB
    from dbutils.persistent_db import PersistentDB
except:
    print("not found model pymysql")


class PyMysqlPooledDB(object):
    """
    链接池
    用于一般应用服务
    """
    __pool = None

    def __init__(self, **kwargs):
        """

        :param mincached:连接池中空闲连接的初始数量
        :param maxcached:连接池中空闲连接的最大数量
        :param maxshared:共享连接的最大数量0，禁止共享链接，确保线程安全
        :param maxconnections:创建连接池的最大数量
        :param blocking:超过最大连接数量时候的表现，为True等待连接数量下降，为false直接报错处理
        :param maxusage:单个连接的最大重复使用次数
        :param setsession:optional list of SQL commands that may serve to prepare
            the session, e.g. ["set datestyle to ...", "set time zone ..."]
        :param reset:how connections should be reset when returned to the pool
            (False or None to rollback transcations started with begin(),
            True to always issue a rollback for safety's sake)
        :param host:数据库ip地址
        :param port:数据库端口
        :param db:库名
        :param user:用户名
        :param password:密码
        :param charset:字符编码
        """

        if not self.__pool:
            cursorclass = {1: pymysql.cursors.DictCursor,
                           2: pymysql.cursors.Cursor}[kwargs.get("cursorclass", 1)]
            self.__class__.__pool = PooledDB(pymysql,       # 默认pymysql.threadsafety=1，确保线程安全。线程可以共享模块，但不能共享连接。
                                             host=kwargs.get("host", "127.0.0.1"),
                                             port=kwargs.get("port", 3306),
                                             user=kwargs.get("user", 'root'),
                                             password=str(kwargs.get("password", "")),
                                             database=kwargs.get("database", "test"),
                                             mincached=kwargs.get("mincached", 10),
                                             maxcached=kwargs.get("maxcached", 20),
                                             maxconnections=kwargs.get("maxconnections", 512),
                                             maxusage=kwargs.get("maxusage", 100),
                                             blocking=kwargs.get("blocking", False),
                                             setsession=kwargs.get("setsession", None),
                                             reset=kwargs.get("reset", True),
                                             charset=kwargs.get("charset", "utf8mb4"),
                                             cursorclass=cursorclass
                                             )

        self.conn = self.__pool.connection(shareable=True)  # shareable=True, 所有线程共享链接.False线程专用，每次调用回分配一个空闲链接，可能不一样
        self.cursor = self.conn.cursor()
        self.autocommit_mode = kwargs.get("autocommit", True)  # True 每次执行自动提交事务；False 需手动调用commit()提交事务

    def select_one(self, sql, params=None):
        """
        :param sql: qsl语句
        :param param: sql参数
        :example:
            select_one("select * from demo where name=%s", params=(2, ))
        """
        self.__execute(sql, params)
        result = self.cursor.fetchone()
        """:type result:dict"""
        result = self.__dict_datetime_obj_to_str(result)
        return result

    def select_all(self, sql, params=None):
        """
        :param sql: qsl语句
        :param param: sql参数
        :return: 结果数量和查询结果集
        :example:
            self.select_all('select * from demo where name=%s', (2,))
        """
        count = self.__execute(sql, params)
        result = self.cursor.fetchall()
        """:type result:list"""
        [self.__dict_datetime_obj_to_str(row_dict) for row_dict in result]
        return result

    def insert(self, table, insert_data, many=False, new=True):
        """
        from pymysql.converters import escape_string
        content: escape_string(content)
        :param table:
        :param insert_data  type:[{"field1": 1},{"field2": 2}]
        :param many  type: true 批量写入（last_insert_id()得到的值不准确）
                           false 逐条写入（last_insert_id()得到的值为准确值,需要获得最新的id值，请使用逐条写入）
                           last_insert_id(): 当前连接，整个数据库内，上一次insert事务提交后，生成的最后一个自增key的值,
                           若连接断开，则值为0
        :param new  type: 若id为自增键时，True 返回最新记录，False返回影响条数
        :return:count 1 影响的行数
        """
        count = 0
        last_insert_ids = []
        insert_datas = insert_data if isinstance(insert_data, (list, tuple)) else [insert_data]
        try:
            columns = ','.join(list(insert_datas[0].keys()))
            params = [tuple(data.values()) for data in insert_datas]
            values = (len(params[0]) * "%s,").rstrip(",")
            sql = "insert into " + table + " (" + columns + ") values (" + values + ")"
            if many:
                count = self.cursor.executemany(sql, params)
            else:
                for item in params:
                    count += self.cursor.execute(sql, item)
                    if new:
                        self.cursor.execute("select last_insert_id() as id")
                        result = self.cursor.fetchone()
                        if result:
                            last_insert_ids.append(result["id"])
            self.__commit()
        except Exception as e:
            self.rollback()
            raise e
        else:
            if new:
                if not isinstance(insert_data, (list, tuple)):
                    sql = f'select * from {table} where id = %s'
                    return self.select_one(sql, last_insert_ids[0])
                else:
                    sql = f'select * from {table} where id in %s'
                    return self.select_all(sql, [tuple(last_insert_ids)])
            else:
                return count

    def inserts(self, table, insert_data, new=False):
        """
        批量写入
        new： True 返回新写入的数据列表（写入为逐条写入，非原子性） False不返回最新数据，只返回影调条数（原子写入）
        """
        if new:
            datas = self.insert(table, insert_data, many=False, new=True)
            return datas
        else:
            counts = self.insert(table, insert_data, many=True, new=False)
            return counts

    def update(self, sql, params=None):
        """
        :param sql:
        :param params:
        :return:
        :example:
            # update all data
            self.update('update demo set name=22 where name=%s', (2,))
            # update first one with order asc
            self.update('update demo set name=2 where name=%s order by id limit 1', (22,))
        """
        count = self.__execute(sql, params)
        return count

    def delete(self, sql, params=None):
        """
        :param sql:
        :param params:
        :return:
        :example:
            # delete all data
            self.delete('delete from demo where name=%s', (22,))
            # delete first one with order asc
            self.delete('delete from demo where name=%s order by id limit 1', (22,))
        """
        count = self.__execute(sql, params)
        return count

    def query_sql(self, sql, params=None):
        """
        查询数据
        sql不要拼接或采用format()，请用传参方式，防止注入

        If args is a list or tuple, %s can be used as a placeholder in the query.
        If args is a dict, %(name)s can be used as a placeholder in the query.

        :param sql:
        :param param:
        :example::
            data = query_sql("select * from demo where name=%s and id=%s", (1, 2))
            data = query_sql("select * from demo where name=%(name)s", {"name": 2})
            data = query_sql("select * from demo where name=name", {"name": 2})
        :return:

        """
        try:
            self.execute(sql, params)
        except Exception as e:
            logging.error(sql)
            raise e
        else:
            result = self.cursor.fetchall()
            return result or []

    def execute(self, sql, params=None):
        """
        提交数据
        原生执行，sql不要拼接.format，用传参方式，防止注入
        常用于：insert、delete、update
        :param sql:
        :param param: [1, 2]、[(1, 2), (3, 4)]
        :return:
        """
        count = 0
        try:
            count = self.__execute(sql, params)
        except Exception as e:
            logging.error(sql)
            self.rollback()
            raise e
        else:
            return count

    def begin(self):
        """开启事务: 先关闭自动提交模式"""
        self.conn.begin()

    def end(self, option='commit'):
        """
        结束事务：
        提交事务或回滚
        还原事务提交模式
        """
        if option == 'commit':
            self.commit()
        else:
            self.rollback()

    def commit(self):
        """手动提交事务"""
        self.conn.commit()

    def rollback(self):
        """
        回滚
        :return:
        """
        self.conn.rollback()

    def close(self):
        try:
            self.cursor.close()
            self.conn.close()
        except Exception as e:
            raise e

    def parse_set_condition_value(self, **params):
        sql_columns = ""
        for k in params.keys():
            sql_columns += k + "=%s,"
        sql_columns = sql_columns.rstrip(",")
        sql_values = params.values()
        return sql_columns, list(sql_values)

    def __commit(self):
        """按当前autocommit配置，决定是否提交事务"""
        if self.autocommit_mode:
            return self.conn.commit()
        else:
            pass    # 需手动调用self.commit()提交

    def __execute(self, sql, params=None):
        count = self.cursor.execute(sql, params)
        self.__commit()
        return count

    @staticmethod
    def __dict_datetime_obj_to_str(result_dict):
        """把字典里面的datatime对象转成字符串"""
        if result_dict:
            result_replace = {k: v.__str__() for k, v in result_dict.items() if isinstance(v, datetime.datetime)}
            result_dict.update(result_replace)
        return result_dict


class PyMysqlPersistentDB(PyMysqlPooledDB):
    """
    用于多线程编程
    线程专用链接模式
    """
    __pool = None

    def __init__(self, **kwargs):
        if not self.__pool:
            cursorclass = {1: pymysql.cursors.DictCursor,
                           2: pymysql.cursors.Cursor}[kwargs.get("cursorclass", 1)]
            self.__class__.__pool = PersistentDB(
                host=kwargs.get("host", "127.0.0.1"),
                port=kwargs.get("port", 3306),
                user=kwargs.get("user",'root'),
                password=str(kwargs.get("password", "")),
                database=kwargs.get("database", "test"),
                creator=pymysql,                                # 使用链接数据库的模块# 默认pymysql.threadsafety=1，确保线程安全。线程可以共享模块，但不能共享连接。
                maxusage=kwargs.get("maxusage", None),          # 一个链接最多被重复使用的次数，None表示无限制
                setsession=kwargs.get("setsession", []),        # 开始会话前执行的命令列表。如：["set datestyle to ...", "set time zone ..."]
                closeable=kwargs.get("closeable", False),       # 如果为False时， conn.close() 实际上被忽略，供下次使用，主线程关闭时才会关闭链接。如果为True时， conn.close()则关闭链接，那么再次调用pool.connection时就会报错。（pool.steady_connection()可以获取一个新的链接）
                threadlocal=kwargs.get("threadlocal", None),    # 本线程独享值得对象，用于保存链接对象，如果链接对象被重置
                charset=kwargs.get("charset", "utf8mb4"),
                cursorclass=cursorclass
            )
        self.conn = self.__pool.connection(shareable=False)    # False不共享链接。 同一个主进程线程/子线程独占链接。不同的线程/子线程间，链接不一样
        self.cursor = self.conn.cursor()
        self.autocommit_mode = kwargs.get("autocommit", True)  # True 每次执行自动提交事务；False 需手动调用commit()提交事务





#!/usr/bin/env python
# -*- coding:utf-8 -*-
import os
from lcyframe.libs.singleton import MongoCon
from model.business_model import BusinessSchema
from model.user_model import UserSchema
from context import InitContext
config = InitContext.get_context()

def init_db():
    db = MongoCon().get_database(**config["mongo_config"])
    if not db.business.count():
        # 初始化商户
        business = vars(BusinessSchema())
        business["name"] = "anhua"
        db.business.insert(business)

        # 新增第一个用户
        from lcyframe.libs.aes import AES
        user = vars(UserSchema())
        user["uid"] = 1
        user["username"] = "anhua"
        user["nickname"] = "安华"
        user["password"] = AES(key=config["aes_config"]["secret"]).encode("000000")
        user["business_id"] = str(business["_id"])
        db.user.insert(user)

def mkdir():
    os.system("mkdir -p ./data/images/head")
    os.system("mkdir -p ./data/bak")
    os.system("mkdir -p ./data/case")
    os.system("mkdir -p ./data/files")
    os.system("mkdir -p ./data/tmp")

if __name__ == "__main__":
    init_db()
    mkdir()


# -*- coding:utf-8 -*-

import json
import base64
from Crypto.Hash import SHA256 as SHA
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5
from requests import session
from lcyframe.libs import utils
import json
import requests
import logging
import time
# s = session()
#
# s.headers.update({"Content-Type": "application/json"})

default_app_id = "201805226573456"

rsa_private_key = """-----BEGIN RSA PRIVATE KEY-----
UVurY12bbXKCAQEAyPApJtxu6iBFlJ9M3l41Mht1FqGOba8IEokEB+fJq8FMdGkp
RCo+axJ1SoDX30+uKcR7u5oOESv7jyYIb4S8j+HcC/OxuWTzqawnpuiLWz4djEKn
n3IwpQIawLPq9wb3B+vxwjWzIqH553axBqu9aHAScEZFJxOfpEKhsk2mnQd7fwE2
g72f9VmgLyNVHnQHjhCa0SswIG6lG0kwOz1bMNSwGyheel8jpiF5qW+QjKG6UsVK
nl2yNECdqsDMTpHXaUYvBIaq4FUVurY12bbXTULy02lmtT3/ieanK2yaeOEpSQRx
VwDhU0Wa4E/NJHMyljG4gfATbaXshMhgrPWsLQIDAQABAoIBAA/hAq+CffVchYMg
L5fnA/coulEGhVXfnhiw4TOquoIrLVqJsOwlFneppsEze1u3VUERuOFzZCcxPZjr
XYFKkQghxhzfrizDcIn5A80p1VFpkDY0UkDEalmv1+NglHHfCAiFOb6qNbTH7hK/
i2/GhJpLOPnMY8yZvWTiqxqHDmKcP+QWXwhV31dwMKfMzdQFb4iLGyjDCZ1F8pm8
hql0kW/fcpiTJTESICC9sAsgtZ+Oiv031e0tqwQOEdfbEVjg1FNuRXMK4vuRKYdC
flrJelDBxAHcog/lSR8i81E9HSGomacyfXSnNxfIJkWHKYytDXwYfyHe8qhDJp87
FhCQu2ECgYEA62PSWGNyTf7byuMEn2iFzoYrxU1lzQ1ewBHz30wQbEROErx7pYe8
JvNdNF3rOH/7T5qdVjQaO13+9TuauQJ/ymLkKIjpUBvlQsHfED2JXaaK7U2f+C4Z
WWL7y3pzchpvqk4KYGRZQxwhr9IdrpeYcLiPyYrWyxiC6MOV60zAhwUCgYEA2ogd
XQdJgdh2BMBCOB3en/Ipjpaxcq3ZrP99iGVm0lTUZR3vBSil/hbGAutJx/7/wpGo
kMNCHrdbtnnhKM1Sf34sM5jpB/+ZC1nrrfmCsClPVvPvfzWQ2e8oB9M1RDKt30fi
KLeAupatYaen29PB4sdwcT7eIz/XfaGTecrWyQkCgYANYCK3uw7nt6+tm7DLjhjs
X1tlXryGJlhX+a0t5xiXzlnRXrx373qVjAajzyJql3skTtjZ6SQEc5blQHnOQYIZ
cf0dAIqhbrRcUr/mIFkJ5UjHqz2H0LbICYPdUBv3Q+FMgBq+13TrB1Iyt3HQVfTn
ktU40pbZ+46Uw2hlcnsOMQKBgQCaJyY/d2J8+8rUsyKDH17piROTh+2IreY9SMSu
sIkFmd1xrtnq06+0OzBW4s7bC6AzCjjUVUiRDlrdr8AK3jtxv/lCNH09rSmyfi3o
PXfY+GsFNXIeVBToBvJAAtcyUmWgeb6pZjiiZqPLnU6lnQzKAlHpMKXWS7Sg5Jd5
dIJqsQKBgF5L2XL5TN6zUcQKIuwhXEMdtzuQEx7BDDvBsJh60Zzura/jJsG78lgm
n9FdQBch5BwSbAhmBTutYXN8r9cfFadW/5nG3GKbelm9cm8GR30gFu47Z9m8svCN
rHdshZW6xMAWmBOlPMwFGPFGh/GaIN2hNy9KENQnT9nKAlHpMKXW
-----END RSA PRIVATE KEY-----
"""

rsa_public_key = """-----BEGIN PUBLIC KEY-----
Qd7fwE2g72kqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAyPApJtxu6iBFlJ9M3l41
Mht1FqGOba8IEokEB+fJq8FMdGkpRCo+axJ1SoDX30+uKcR7u5oOESv7jyYIb4S8
j+HcC/OxuWTzqawnpuiLWz4djEKnn3IwpQIawLPq9wb3B+vxwjWzIqH553axBqu9
aHAScEZFJxOfpEKhsk2mnQd7fwE2g72f9VmgLyNVHnQHjhCa0SswIG6lG0kwOz1b
MNSwGyheel8jpiF5qW+QjKG6UsVKnl2yNECdqsDMTpHXaUYvBIaq4FUVurY12bbX
TULy02lmtT3/ieanK2yaeOEpSQRxVwDhU0Wa4E/NJHMyljG4gfATbaXshMhgrPWN
JHMyljG4gf
-----END PUBLIC KEY-----
"""

url = "https://openapi.alipay.com/gateway.do"


class AlipayOpenApi(object):
    """
    ******
    """

    def __init__(self, app_id=None,
                 private_path=None,
                 private_string=None,
                 public_path=None,
                 public_string=None):

        self.app_id = app_id or default_app_id
        self.private_path = private_path
        self.private_string = private_string or rsa_private_key
        self.public_path = public_path
        self.public_string = public_string or rsa_public_key

        self._load_key()

    def _load_key(self):
        # load private key
        content = self.private_string
        if not content:
            with open(self.private_path) as fp:
                content = fp.read()
        self.private_key = RSA.importKey(content)

        # load public key
        content = self.public_string
        if not content:
            with open(self.public_path) as fp:
                content = fp.read()
        self.public_key = RSA.importKey(content)

    def sign(self, sign_string):
        """
        ******
        :param sign_data:
        :param private_key:
        :return:
        """
        # signer = PKCS1_v1_5.new(RSA.importKey(self.private_key))
        signer = PKCS1_v1_5.new(self.private_key)
        return base64.b64encode(signer.sign(SHA.new(sign_string)))

    def get_sign(self, **params):
        """
        ******
        :param data:
        :return:
        """
        mapping = {}
        for k, v in params.items():
            mapping[k.lower()] = v

        keys = [k.lower() for k in mapping.keys()]
        keys.sort()

        sign_string = "&".join(["%s=%s" % (k, mapping[k]) for k in keys if mapping[k] != ""])

        return self.sign(sign_string)

    def get_token(self, auth_code):
        """
        auth_code******access_token
        :param auth_code:
        :return:
        """
        params = {
            "app_id": self.app_id,
            "method": "alipay.open.auth.token.app",
            "charset": "utf-8",
            "sign_type": "RSA2",
            # "sign": "",
            "timestamp": utils.int_to_date_string(utils.now(), "%Y-%m-%d %H:%M:%S"),
            "version": "1.0",
            "grant_type": "authorization_code",
            "code": auth_code,
            "biz_content": "{}"
        }

        params["sign"] = self.get_sign(**params)
        # params["sign_type"] = "RSA2".encode("u8")

        data = requests.post(url, params)
        content = json.loads(data.text)
        pass

    def get_precreate_qrcode(self, **kwargs):
        """
        ******，******
        :return:
        """
        biz_content = {
            "out_trade_no": kwargs["demo_order_id"],
            "total_amount": str("%.02f" % (kwargs["need_amount"] / 100.)),
            "subject": "******"  # ******
        }
        params = {
            "app_id": self.app_id,
            "method": "alipay.trade.precreate",
            "charset": "utf-8",
            "sign_type": "RSA2",
            "timestamp": utils.int_to_date_string(utils.now(), "%Y-%m-%d %H:%M:%S"),
            "version": "1.0",
            "notify_url": "%s/face2face_nodify?cpoi_xwr=%s" % (kwargs["host"], kwargs["cpoi_xwr"]),
        }

        params["biz_content"] = json.dumps(biz_content)
        params["sign"] = self.get_sign(**params)
        data = requests.post(url, params, timeout=10)
        if data.status_code == 200:
            content = json.loads(data.text)
            if content["alipay_trade_precreate_response"]["msg"] == "Success":
                return content["alipay_trade_precreate_response"]["qr_code"]
            else:
                logging.warning("******：%s ******：%s" % (self.app_id, content))
        else:
            logging.warning("******：%s ******：%s" % (self.app_id, str(data.status_code)))
            return ""

    def get_precreate_qrcode2(self, **kwargs):
        """
        ******，******
        :return:
        """
        biz_content = {
            "out_trade_no": kwargs["demo_order_id"],
            "total_amount": str("%.02f" % (kwargs["need_amount"] / 100.)),
            "subject": "******"  # ******
        }
        params = {
            "app_id": self.app_id,
            "method": "alipay.trade.wap.pay",
            "charset": "utf-8",
            "sign_type": "RSA2",
            "timestamp": utils.int_to_date_string(utils.now(), "%Y-%m-%d %H:%M:%S"),
            "version": "1.0",
            "notify_url": "%s/native_nodify?cpoi_xwr=%s" % (kwargs["host"], kwargs["cpoi_xwr"]),
        }

        params["biz_content"] = json.dumps(biz_content)
        params["sign"] = self.get_sign(**params)
        data = requests.post(url, params, timeout=10)
        if data.status_code == 200:
            qrcode_url = data.url
            if qrcode_url:
                return data.url
            else:
                logging.warning("******：%s ******：%s" % (self.app_id, utils.to_json(data.content)))
        else:
            logging.warning("******：%s ******：%s" % (self.app_id, str(data.status_code)))
            return ""


if __name__ == "__main__":
    api = AlipayOpenApi()

    params = {
        "app_id": "2014072300007148",
        "charset": "UTF-8",
        "sign_type": "RSA2",
        "timestamp": "2018-12-12 19:45:38",  # utils.int_to_date_string(utils.now(), "%Y-%m-%d %H:%M:%S"),
        "version": 1.0
    }

    # print(api.get_sign(**params) == "y7JKfSzSuvPMmqAmhIAiTpPGLmt6WA1uKS+S/E12p8Dev+reBgZStfvRqQ83ASAc+K3wh96ONBZcqSGGzVnU5xt8n3QRi8OCW7jll8XE5XBPcmdvYn/5w6gcU1utQlKu8fM8lUtq5h04XQt44I7pWUMz7d5AhZBvTCEL4h+30X7TIvpYqzLRd1n3Xk1fxpFgurhdqGq3QOuWNQljNYcZzKxGeBzV1D54O9s2/F6sVIDhAAdagB5LpvrzhXvjk3ZVHFgjxWSHJNFP0Xo4VGBvnUWQYWtpqnRH6FFZDJUq8Jws6ZIgiqX97gA4UZ2eBVq8KjvB9sqo1aor6m49eNjz0g==")

    # ******access_token
    # print(api.get_token("4b203fe6c11548bcabd8da5bb087a83b"))

    # ******
    params = {
        "demo_order_id": int(time.time()),
        "need_amount": 2,
        "host": "http://open.cplife.club",
        "cpoi_xwr": "111"
    }
    # print(api.get_precreate_qrcode(**params))
    print(api.get_precreate_qrcode2(**params))



# coding=utf-8
import os
from json import loads, dumps
import logging
from lcyframe.libs import utils

class NsqTask(object):
    """订阅队列"""
    _workers = {}

    def __init__(self, topic, channel):
        self.topic = topic
        self.channel = channel

    def __call__(self, _handler):
        # name = self.name and self.name or _handler.__name__
        setattr(_handler, "topic", self.topic)
        setattr(_handler, "channel", self.channel)
        self._workers[_handler.__name__] = _handler
        return _handler

    @classmethod
    def get_workers(cls, ROOT, workers_path):
        if not ROOT:
            raise Exception("the project dir path must been give， and None.")

        if not isinstance(workers_path, list):
            workers_path = [workers_path]

        if workers_path is None:
            raise Exception("nsq workers_path is not allow empty")

        if not cls._workers:
            for work in workers_path:
                for root, dirs, files in os.walk(utils.fix_path(os.path.join(ROOT, work))):
                    for file in files:
                        if file.startswith("__"):
                            continue
                        if file.endswith(".pyc"):
                            continue
                        if not file.endswith(".py"):
                            continue

                        model_name = root.replace(ROOT, "").lstrip("/").replace("/", ".") + "." + file.rstrip(".py")
                        __import__(model_name, globals(), locals(), [model_name], 0)
                        logging.debug("register nsq workers [%s.py] success!" % model_name)
        return cls._workers


class ReadNsq(object):
    """
    消费者：当接收新消息后调用该方法
    """

    @classmethod
    def message_handler(cls, message):
        msg = loads(message.body.decode("utf-8"))
        event = msg.pop("event", None)

        if not hasattr(cls, event):
            logging.warning(dumps(msg))
            logging.warning("has a event '%s' not realized in Class %s" % (event, cls.__name__))
            return True

        return getattr(cls, event)(**msg)

class WriteNsq(object):
    """
    生产者：向已订阅的topic里发送消息
    """
    _nsq = None

    def __init__(self, nsq_conn=None):
        if nsq_conn is None:
            try:
                from .singleton import NsqCon
                nsq_conn = NsqCon.get_connection()
            except:
                raise Exception("your must register nsq conn before start.")

        self._nsq = nsq_conn

    def __getattr__(self, name):
        # if name.startswith('__'):
        #     raise AttributeError("type object '%s' has no attribute '%s'" %
        #                          (self.__class__.__name__, name))
        if hasattr(self.__class__, '__events__'):
            if name not in self.__class__.__events__:
                raise Exception("Event '%s' is not exists in your Producer Class" % name)

        if name not in self.__dict__:
            self.__dict__[name] = pub = AgentNsq(self._nsq, name, self.topic)
            return pub
        else:
            return self.__dict__[name]

class AgentNsq:
    def __init__(self, _nsq, event, topic):
        self.event = event
        self.topic = topic
        self._nsq = _nsq

    def __repr__(self):
        return "event '%s'" % self.event

    def __call__(self, kwargs):
        kwargs["event"] = self.event        # message_handler name by Consumer
        self._nsq.pub(self.topic, dumps(kwargs).encode("utf-8"))





#!/usr/bin/env python
# -*- coding:utf-8 -*-
from lcyframe import BaseModel
from model.schema.user_schema import UserSchema


class UserModel(BaseModel, UserSchema):

    @classmethod
    def get(cls, *args, **kwargs):
        """
        单条记录
        :return:
        :rtype:
        """
        pass

    @classmethod
    def query(cls, *args, **kwargs):
        """
        列表
        :return:
        :rtype:
        """
        pass

    @classmethod
    def create(cls, *args, **kwargs):
        """
        创建
        :return:
        :rtype:
        """
        pwd = kwargs.pop("pass_word")

        if not pwd or not kwargs["user_name"]:
            return -1

        if cls.find_one({"nick_name": kwargs["nick_name"]}):
            return - 2

        docs = {}
        docs["gid"] = kwargs["gid"]
        docs["uid"] = cls.IdGeneratorModel.gen_uid_id()
        docs["salt"] = cls.utils.gen_salt()
        docs["pass_word"] = cls.utils.gen_salt_pwd(pwd, docs["salt"])
        docs.update(kwargs)
        cls.insert(docs)
        return docs["uid"]

    @classmethod
    def modify(cls, *args, **kwargs):
        """
        修改
        :return:
        :rtype:
        """
        spec = {"uid": kwargs.pop("uid")}
        update_docs = {}
        for k, v in kwargs.items():
            if k not in vars(cls.UserSchema()):
                continue

            # 改密码
            if k == "pass_word" and kwargs["pass_word"]:
                update_docs[k] = cls.utils.gen_salt_pwd(kwargs["pass_word"], kwargs["salt"])
            # 改用户组
            elif k == "gid":
                update_docs[k] = v
                update_docs["permission"] = cls.UserSchema.gid_permission[v]
            else:
                update_docs[k] = v

        return cls.update(spec, update_docs)

    @classmethod
    def delete(cls, *args, **kwargs):
        """
        删除
        :return:
        :rtype:
        """
        pass




import os
import logging
from logging import handlers

"""
https://www.cnblogs.com/nancyzhu/p/8551506.html
https://www.cnblogs.com/xianyulouie/p/11041777.html

参数：作用
 
%(levelno)s：打印日志级别的数值
%(levelname)s：打印日志级别的名称
%(pathname)s：打印当前执行程序的路径，其实就是sys.argv[0]
%(filename)s：打印当前执行程序名
%(funcName)s：打印日志的当前函数
%(lineno)d：打印日志的当前行号
%(asctime)s：打印日志的时间
%(thread)d：打印线程ID
%(threadName)s：打印线程名称
%(process)d：打印进程ID
%(message)s：打印日志信息

"""

level_mp = {
        'debug': logging.DEBUG,
        'info': logging.INFO,
        'warning': logging.WARNING,
        'error': logging.ERROR,
        'critical': logging.CRITICAL
    }

default_fmt = '%(asctime)s - %(pathname)s:%(lineno)d - %(levelname)s: %(message)s'
lcy_format = '[{name}] %(levelname)s\n[{name}] path: %(pathname)s:%(lineno)d\n[{name}] time: %(asctime)s\n[{name}] message: %(message)s'

class Logger(object):
    instance = {}

    def __init__(self, name, level='info', when='D', backCount=10, fmt=None, save_path=None, propagate=True):
        if not fmt:
            fmt = lcy_format.format(name=name)
        else:
            fmt = default_fmt
        self.propagate = propagate
        self.logger = logging.getLogger(name)
        self.logger.propagate = self.propagate           # 输出的信息不推送给上级的logger，即root
        self.format_str = logging.Formatter(fmt)
        self.logger.setLevel(level_mp[level])

        # redirct to console
        console = logging.StreamHandler()
        console.setFormatter(self.format_str)
        self.logger.addHandler(console)

        if save_path and not os.path.isabs(save_path):
            logging.error("save_path require a path like this '/root/my_project/out.log'")
            exit()

        # wirte to file
        if save_path:
            handler = handlers.TimedRotatingFileHandler(filename=save_path,       # 保存路径
                                                        when=when,              # 间隔的时间单位，单位有: S秒，M 分，H 小时，D 天，W 每星期（interval==0时代表星期一），midnight 每天凌晨
                                                        backupCount=backCount,  # 文件的个数，如果超过这个个数，就会自动删除
                                                        encoding='utf-8')
            handler.setFormatter(self.format_str)
            self.logger.addHandler(handler)


    @classmethod
    def get_instance(cls, name="Lcyframe", **kwargs):
        if name not in cls.instance:
            cls.instance[name] = Logger(name, **kwargs)
        return cls.instance[name]

if __name__ == '__main__':
    # app 专用log
    app = Logger.get_instance(name="Lcyframe", level='debug')
    app.logger.debug('debug')
    app.logger.info('info')
    app.logger.warning('警告')
    app.logger.error('报错')
    app.logger.critical('严重')
    #
    # 数据库专用log
    # db = Logger('db', level='error', fmt=default_fmt)
    # db.logger.error('error')


# -*- coding:utf-8 -*-
from lcyframe.libs import Copyright

def get_handler_init(path):
    """

    :param path:
    :return:
    """

    auto_str = """%s
import os
import re
import logging
dir_path = os.listdir(os.path.abspath(os.path.dirname(__file__)))

""" % (Copyright.file_pre)

    import_str = """
# xxx_handler xx leng must > 1.
hander_files = [x for x in dir_path if re.findall('[A-Za-z]\w+\.py$', x)]

for hander_file in hander_files:
    model_name = hander_file[:-3]
    if model_name == "__init__":
        continue
    logging.debug("handler [%s] register success!" % hander_file)
    __import__(model_name, globals(), locals(), [model_name], -1)
"""

    return auto_str + import_str


def get_docs_agree():
    """

    :return:
    """
    auto_str = """
## 一、约定说明
### 1.1 参数说明

**请求方式：本框架规定了所有请求参数所配置的方式以及用途：**

|参数位置|Content-Type|举个例子|说明|
|---|---|---|---|
|path|--|/url/{user}/{uid}|拼接在url内|
|query|--|/url/user?uid=10|拼接在?后|
|body|application/x-www-form-urlencoded|{"uid": 100, "sex": 1}|表单提交，推荐|
|body|multipart/form-data|{"file": open("demo.jpg"), "header": open("header.jpg")}|上传文件|
|body|application/json|'uid=100&sex=1'或'{"uid":100,"sex:1"}'|json提交|
|body|text/plain|'uid=100&sex=1'或'{"uid":100,"sex:1"}'|--|

> 1、path：参数被设置在请求url内；如"/url/xxx/yyy"。注意，使用该模式，框架不会自动收集参数，即self.params永远为空

> 2、query：参数被设置在?后面：如"/url?xxx=111&yyy=222"

> 3、body: json提交，参数以json的格式被设置在body内，body设置为x-www-form-urlencoded，Content-Type设置为"application/json"。注意该模式与body-form不能同时设置。

> 4、body：表单提交，参数以kv的格式被设置在body内，body设置为x-www-form-urlencoded，Content-Type设置为"application/x-www-form-urlencoded"。注意该模式与body-json不能同时设置。

> 5、file：上传文件。body设置为form-data，Content-Type设置为"multipart/form-data;boundary=abcefxxx",boundary分解符通常会自动被设置为一个随机字符 
> 
> 注意：以上345模式，由于值都被设置在body内，所以在同一api内，对于不同的参数，不允许混用不同的模式。唯一的例外是，第5（multipart/form-data），既允许同时上传文件，又允许设置其他参数。

**Restful请求示例：**
~~~python
### Send POST request with body as form parameters
POST https://domain.com/post
Content-Type: application/x-www-form-urlencoded

{
  "id": 999,
  "value": "content"
}

### Send POST request with json body
POST https://domain.com/post
Content-Type: application/json

id=999&value=content

### Send a form with the text and file fields
POST https://domain.com/post
Content-Type: multipart/form-data; boundary=WebAppBoundary

--WebAppBoundary
Content-Disposition: form-data; name="element-name"
Content-Type: text/plain

Name
--WebAppBoundary
Content-Disposition: form-data; name="data"; filename="data.json"
Content-Type: application/json

< ./request-form-data.json
--WebAppBoundary--

###
~~~

**参数值类型：在设置schema.yml时，指定type的值允许是以下几种之一**

|类型|说明|
|---|---|
| int | 整形|
| float | 浮点型|
| str | 字符串|
| list | 数组（不推荐)|
| json | json串|
| file | 文件|

### 1.2 headers
API请求时，需要提供请求头

headers：
~~~python
{
    "uid": "10000，注册成功、登录成功后返回",
    "token": "授权码, 注册成功、登录成功后返回",
    "locale": "cn",
    "lang": "",
    "version": "默认 1.0", 
    "bundle": "",
    "platform": 0 IOS, 1 Android,
    "os": "",
    "device_id": ""
}
~~~
### 1.3 API 请求结果
当`HTTP_CODE=200`时，代表本次请求响应成功，响应结构如下所示：
~~~python
{
  "error": 0, 
  "msg": msg, 
  "data": {}
}
~~~

>`error`: 当`error=0`时，代表API处理成功，客户端可以正常处理data；当`error!=0`时，代表API处理失败，error即为对应的异常码
>
>`msg`: 字段为本次请求的提示信息。当`error=0`时，`msg=ok`；当`error!=0`时，`msg=ErrorName`
>
>`data`: 为本次API返回的数据，如{'nickname'：'昵称', 'sex'：1}。在接口文档中，不同的API所返回的结构不同,具体详见《接口说明》

当`HTTP_CODE!=200`时，如`404`、`405`、`500`的错误码，客户端按需处理

### 1.4 双向校验码seq生成
部分防刷接口需要提供参数seq，生成方法如下

salt：写死在客户端
~~~
例如：
params={"nickname": "阿刘", "age": "111", "timestamp": 1473761433}
上述2个参数value 均为 utf-8 编码, 根据key的字典顺序排序 键和值之间用"="连接 键值对之间用"&"连接,
生成待校验的字符串string, string 与 salt 字符串 拼接,计算校验码：
seq = md5(salt + "nickname=阿刘&age=111&timestamp=1473761433")
注：凡需要传seq的接口，必须同时提供请求时间戳参数：timestamp
~~~

### 1.5 翻页码规则：仅支持查看下一页，不支持跳页
参数名：last_id，由服务器返回。客户端可以在请求列表时，回传给服务端，
~~~
    1、当不提供该参数时，代表查看第一页
    2、当服务端返回last_id=-1时，代表没有下一页了
~~~

### 1.6 域名配置
**本地环境：**
> 1、api：http://192.168.2.50:6700  
> 2、docs文档：http://192.168.2.50:6779

**测试环境：**
> 1、api：http://192.168.2.140:6700  
> 2、docs文档：http://192.168.2.140:6779

### 1.7、免责声明
本框架仅能在取得足够合法授权的企业安全建设中使用，在使用本过程中，您应确保自己所有行为符合当地的法律法规。 
如您在使用本工具的过程中存在任何非法行为，您将自行承担所有后果，开发者不承担任何法律及连带责任。 
除非您已充分阅读、完全理解并接受本协议所有条款，否则，请您不要安装并使用本框架。
您的使用行为或者您以其他任何明示或者默示方式表示接受本协议的，即视为您已阅读并同意本协议的约束。

"""
    return auto_str


def get_mq_work_init():
    auto_str = """%s
import os
import re
import logging
dir_path = os.listdir(os.path.abspath(os.path.dirname(__file__)))
    """ % Copyright.file_pre

    import_str = """
hander_files = [x for x in dir_path if re.findall('[A-Za-z]\w+_worker\.py$', x)]

for hander_file in hander_files:
    model_name = hander_file[:-3]
    logging.debug("register mq workers [%s.py] success!" % model_name)
    __import__(model_name, globals(), locals(), [model_name], -1)
    """

    return auto_str + import_str


def get_nsq_producer_init(root_path, produce_path):
    """

    :param path:
    :return:
    """

    auto_str = """%s
import os
import re
import logging
from importlib import import_module
from lcyframe.libs.singleton import NsqCon
nsq = NsqCon.get_connection()

dir_path = os.listdir(os.path.abspath(os.path.dirname(__file__)))
produce_path = "%s"
""" % (Copyright.file_pre, produce_path)

    import_str = """
# xxx_handler xx leng must > 1.
hander_files = [x for x in dir_path if re.findall('[A-Za-z]\w+\.py$', x)]

for hander_file in hander_files:
    model_name = hander_file[:-3]
    if model_name == "__init__":
        continue

    model_name = produce_path.strip("/").replace("/", ".") + "." + model_name
    objs = __import__(model_name, fromlist=["*"])
    for p in dir(objs):
        if p.startswith("__"):
            continue
            
        # if not p.lower().startswith("publish"):
            #logging.debug("nsq producer [%s] load fail. it must been start with 'publish'" % p)
        #    continue
        
        if not hasattr(getattr(objs, p), "topic"):
            continue
        
        logging.debug("nsq producer [%s] register success!" % p)
        setattr(nsq, p, getattr(objs, p)())
"""

    return auto_str + import_str


def get_nsq_work_init():
    auto_str = """%s
import os
import re
import logging
dir_path = os.listdir(os.path.abspath(os.path.dirname(__file__)))
    """ % Copyright.file_pre

    import_str = """
hander_files = [x for x in dir_path if re.findall('[A-Za-z]\w+_worker\.py$', x)]

for hander_file in hander_files:
    model_name = hander_file[:-3]
    logging.debug("register nsq workers [%s.py] success!" % model_name)
    __import__(model_name, globals(), locals(), [model_name], -1)

    """
    return auto_str + import_str


def get_mqtt_producer_init(root_path, produce_path):
    """

    :param path:
    :return:
    """

    auto_str = """%s
import os
import re
import logging
from importlib import import_module
from lcyframe.libs.singleton import MqttCon
mqtt = MqttCon.get_connection()

dir_path = os.listdir(os.path.abspath(os.path.dirname(__file__)))
produce_path = "%s"
""" % (Copyright.file_pre, produce_path)

    import_str = """
# xxx_handler xx leng must > 1.
hander_files = [x for x in dir_path if re.findall('[A-Za-z]\w+\.py$', x)]

for hander_file in hander_files:
    model_name = hander_file[:-3]
    if model_name == "__init__":
        continue

    model_name = produce_path.strip("/").replace("/", ".") + "." + model_name
    objs = __import__(model_name, fromlist=["*"])
    # objs = import_module(model_name)  # 重名py文件会被覆盖
    for p in dir(objs):
        if p.startswith("__"):
            continue
            
        # if not p.lower().startswith("publish"):
            #logging.debug("mqtt producer [%s] load fail. it must been start with 'publish'" % p)
        #    continue
        
        if not hasattr(getattr(objs, p), "topic"):
            continue
            
        logging.debug("mqtt producer [%s] register success!" % p)
        setattr(mqtt, p, getattr(objs, p)())
"""

    return auto_str + import_str


def get_mqtt_work_init():
    auto_str = """%s
import os
import re
import logging
dir_path = os.listdir(os.path.abspath(os.path.dirname(__file__)))
    """ % Copyright.file_pre

    import_str = """
hander_files = [x for x in dir_path if re.findall('[A-Za-z]\w+_worker\.py$', x)]

for hander_file in hander_files:
    model_name = hander_file[:-3]
    logging.debug("register mqtt workers [%s.py] success!" % model_name)
    __import__(model_name, globals(), locals(), [model_name], -1)

    """
    return auto_str + import_str

def get_celery_works_init(root_path, workers_path):
    """

    :param path:
    :return:
    """

    auto_str = """%s
import os
import re
import logging
from lcyframe.libs.singleton import CeleryCon
from lcyframe.libs.celery_route import Events
app = CeleryCon._app

dir_path = os.listdir(os.path.abspath(os.path.dirname(__file__)))
workers_path = "%s"
""" % (Copyright.file_pre, workers_path)

    import_str = """
hander_files = [x for x in dir_path if re.findall('[A-Za-z]\w+\.py$', x)]

for hander_file in hander_files:
    model_name = hander_file[:-3]
    if model_name == "__init__":
        continue

    model_name = workers_path.strip("/").replace("/", ".") + "." + model_name
    objs = __import__(model_name, fromlist=["*"])
    module_name = objs.__name__.split(".")[-1]

    setattr(Events, module_name, objs)

    for p in dir(objs):
        if p.startswith("__"):
            continue

        if type(getattr(objs, p)) == type:
            if not hasattr(getattr(objs, p), "queue"):
                continue

            if hasattr(app, p) and p != 'BaseEvent':
                raise Exception("exists same attribute：%s" % p)

        # logging.debug("celery task [%s] register success!" % p)
        setattr(Events, p, getattr(objs, p))

    setattr(app, Events.__name__, Events)

"""

    return auto_str + import_str


# coding=utf-8
import json
from time import time
from tornado import gen, ioloop
from tornado.httpclient import AsyncHTTPClient, HTTPError


JSON_HEADER = {'content-type': 'application/json'}

def get_async_client():
    return AsyncHTTPClient()


def parse_appkey(appkey):
    """解析appkey, 从中得到org和app, 注意, appkey的规则是 {org}#{app}"""
    return tuple(appkey.split('#'))


class Token:
    """表示一个登陆获取到的token对象"""

    def __init__(self, token, exipres_in):
        self.token = token
        self.exipres_in = exipres_in + int(time())

    def is_not_valid(self):
        """这个token是否还合法, 或者说, 是否已经失效了, 这里我们只需要
        检查当前的时间, 是否已经比或者这个token的时间过去了exipreis_in秒

        即  current_time_in_seconds < (expires_in + token_acquired_time)
        """
        return time() > self.exipres_in


class EasemobAuth(object):
    """环信登陆认证的基类"""

    @gen.coroutine
    def get_headers(self):
        token = yield self.get_token()
        headers = {'Authorization': 'Bearer ' + token,
                   'content-type': 'application/json'}

        raise gen.Return(headers)

    @gen.coroutine
    def get_token(self):
        """在这里我们先检查是否已经获取过token, 并且这个token有没有过期"""
        if (self.token is None) or (self.token.is_not_valid()):
            self.token = yield self.acquire_token()  # refresh the token
        raise gen.Return(self.token.token)

    @gen.coroutine
    def acquire_token(self):
        """真正的获取token的方法, 返回值是一个我们定义的Token对象
            这个留给子类去实现
        """
        pass


class AppClientAuth(EasemobAuth):
    """使用app的client_id和client_secret来获取app管理员token"""

    def __init__(self, org, app, client_id, client_secret, host):
        super(AppClientAuth, self).__init__()
        self.client_id = client_id
        self.client_secret = client_secret
        self.url = host + ("/%s/%s/token" % (org, app))
        self.token = None

    @gen.coroutine
    def acquire_token(self):
        """
        使用clieng_id / client_secret来获取token, 具体的REST API为

        POST /{org}/{app}/token {'grant_type':'client_credentials', 'client_id':'xxxx', 'client_secret':'xxxxx'}
        """
        payload = {'grant_type': 'client_credentials', 'client_id': self.client_id, 'client_secret': self.client_secret}
        response = yield get_async_client().fetch(self.url, method='POST', body=json.dumps(payload))
        if response.code == 200:
            result = json.loads(response.body)
            raise gen.Return(Token(result['access_token'], result['expires_in']))
        else:
            pass


class AppAdminAccountAuth(EasemobAuth):
    """使用app的管理员账号和密码来获取token"""

    def __init__(self, org, app, username, password, host):
        super(AppAdminAccountAuth, self).__init__()
        self.username = username
        self.password = password
        self.url = host + ("/%s/%s/token" % (org, app))
        self.token = None

    @gen.coroutine
    def acquire_token(self):
        """
        使用 username / password 来获取token, 具体的REST API为

        POST /{org}/{app}/token {'grant_type':'password', 'username':'xxxx', 'password':'xxxxx'}

        这里和上面使用client_id不同的是, grant_type的类型是password, 然后还需要提供username和password
        """
        payload = {'grant_type': 'password', 'username': self.username, 'password': self.password}
        response = yield get_async_client().fetch(self.url, method='POST', body=json.dumps(payload))
        if response.code == 200:
            result = json.loads(response.body)
            raise gen.Return(Token(result['access_token'], result['expires_in']))
        else:
            pass


class OrgAdminAccountAuth(EasemobAuth):
    """使用org的管理员账号和密码来获取token,
    和上面不同的是, 这里获取的是整个org的管理员账号,
    所以并没有appkey的概念

    并且, 因为没有appkey的概念, 所以, URL也不相同,
    这里使用的URL是 https://a1.easemob.com/management/token

    而app级别的token都是从 https://a1.easemob.com/{org}/{app}/token
    这个URL去获取的
    """

    def __init__(self, username, password, host):
        super(OrgAdminAccountAuth, self).__init__()
        self.username = username
        self.password = password
        self.url = host + "/management/token"
        self.token = None

    @gen.coroutine
    def acquire_token(self):
        """
        使用 username / password 来获取token, 具体的REST API为

        POST /management/token {'grant_type':'password', 'username':'xxxx', 'password':'xxxxx'}
        """
        payload = {'grant_type': 'password', 'username': self.username, 'password': self.password}
        response = yield get_async_client().fetch(self.url, method='POST', body=json.dumps(payload))
        if response.code == 200:
            result = json.loads(response.body)
            raise gen.Return(Token(result['access_token'], result['expires_in']))
        else:
            pass


class EMChatAsync(object):
    """环信"""

    def __init__(self, **kwargs):
        self.EASEMOB_HOST = kwargs.get("host", "https://a1.easemob.com")
        self.app_client_auth = None
        self.app = kwargs["app"]
        self.org = kwargs["org"]
        self.appkey = self.app + "#" + self.org
        self.org_admin_username = kwargs['username']
        self.org_admin_password = kwargs['password']
        self.client_id = kwargs['client_id']
        self.client_secret = kwargs['client_secret']
        self.app_admin_username = kwargs['username']
        self.app_admin_password = kwargs['password']
        self.PUSH_USER_ID = kwargs.get("push_user_id", "system")
        
    def get_org_admin_auth(self):
        """org_admin的认证方式"""
        return OrgAdminAccountAuth(self.org_admin_username, self.org_admin_password, self.EASEMOB_HOST)

    def get_app_admin_auth(self):
        """org_admin的认证方式"""
        return AppAdminAccountAuth(self.org, self.app, self.app_admin_username, self.app_admin_password, self.EASEMOB_HOST)

    @gen.coroutine
    def get_app_client_headers(self):
        """通过client id和secret来获取app管理员的token"""
        if self.app_client_auth is None:
            self.app_client_auth = AppClientAuth(self.org, self.app, self.client_id, self.client_secret, self.EASEMOB_HOST)

        headers = yield self.app_client_auth.get_headers()
        raise gen.Return(headers)

    @gen.coroutine
    def register_new_user(self, username, password):
        """注册新用户
        """
        data = {"username": username, "password": password}
        url = self.EASEMOB_HOST + ("/%s/%s/users" % (self.org, self.app))
        headers = yield self.get_app_client_headers()

        try:
            response = yield get_async_client().fetch(url,
                                                      method='POST',
                                                      body=json.dumps(data),
                                                      headers=headers)
            if response.code == 200:
                raise gen.Return(response.body)

        except HTTPError as e:
            if e.code == 400:
                # 新增用户、但是用户已经存在了, 不做处理
                if password == "password":
                    raise gen.Return(True)

                # 用户存在, 修改密码
                url = self.EASEMOB_HOST + ("/%s/%s/users/%s/password" % (self.org, self.app, username))
                response = yield get_async_client().fetch(url,
                                                      method='PUT',
                                                      body=json.dumps({"newpassword": password}),
                                                      headers=headers)
                if response.code == 200:
                    raise gen.Return(response.body)

            else:
                # 其他错误
                raise gen.Return(e.response.body)

    @gen.coroutine
    def delete_user(self, username):
        """删除用户
        """
        url = self.EASEMOB_HOST + ("/%s/%s/users/%s" % (self.org, self.app, username))

        headers = yield self.get_app_client_headers()
        response = yield get_async_client().fetch(url,
                                                  method='DELETE',
                                                  headers=headers)
        if response.code == 200:
            raise gen.Return(response.body)

    @gen.coroutine
    def send_file(self, filename):
        """发送文件
        """
        url = self.EASEMOB_HOST + ("/%s/%s/chatfiles" % (self.org, self.app))
        files = {'file': ('report.xls', open(filename, 'rb', encoding='utf-8'), 'image/jpeg', {'Expires': '0'})}

        headers = yield self.get_app_client_headers()
        response = yield get_async_client().fetch(url,
                                                  method='POST',
                                                  body=json.dumps(files),
                                                  headers=headers)
        if response.code == 200:
            raise gen.Return(response.body)

    @gen.coroutine
    def send_msg(self, msg, to_ids, from_user_id=None, ext={}):
        url = self.EASEMOB_HOST + ("/%s/%s/messages" % (self.org, self.app))
        if not isinstance(to_ids, list):
            to_ids = [to_ids]

        data = {"target_type": "users",
                "target": to_ids,
                "msg": {"type": "txt",
                        "msg": msg},
                "from": from_user_id or self.PUSH_USER_ID,
                "ext": ext if isinstance(ext, dict) else json.loads(ext)}

        headers = yield self.get_app_client_headers()
        response = yield get_async_client().fetch(url,
                                                  method='POST',
                                                  body=json.dumps(data),
                                                  headers=headers)
        if response.code == 200:
            raise gen.Return(response.body)

    @gen.coroutine
    def send_ninja_msg(self, msg, to_ids, from_user_id=None, ext={}, target_type="users"):
        url = self.EASEMOB_HOST + ("/%s/%s/messages" % (self.org, self.app))
        if not isinstance(to_ids, list):
            to_ids = [to_ids]

        data = {"target_type": target_type,
                "target": to_ids,
                "msg": {"type": "cmd",
                        "action": msg},
                "from": from_user_id or self.PUSH_USER_ID,
                "ext": ext if isinstance(ext, dict) else json.loads(ext)}
        headers = yield self.get_app_client_headers()
        response = yield get_async_client().fetch(url,
                                                  method='POST',
                                                  body=json.dumps(data),
                                                  headers=headers,
                                                  request_timeout=15)
        if response.code == 200:
            print(str(data).decode('string_escape'))
            raise gen.Return(response.body)


    @gen.coroutine
    def send_loc(self, to_ids, from_user_id=None, addr=None, lat=None, lng=None, ext={}):
        url = self.EASEMOB_HOST + ("/%s/%s/messages" % (self.org, self.app))
        if not isinstance(to_ids, list):
            to_ids = [to_ids]

        data = {"target_type": "users",
                "target": to_ids,
                "msg": {"type": "loc",
                        "addr": addr,
                        "lat": lat,
                        "lng": lng},
                "from": from_user_id or self.PUSH_USER_ID,
                "ext": ext if isinstance(ext, dict) else json.loads(ext)}

        headers = yield self.get_app_client_headers()
        response = yield get_async_client().fetch(url,
                                                  method='POST',
                                                  body=json.dumps(data),
                                                  headers=headers)
        if response.code == 200:
            raise gen.Return(response.body)

    @gen.coroutine
    def send_pic_msg(self, to_ids, from_user_id=None, img_url=None, filename=None, ext={}):
        url = self.EASEMOB_HOST + ("/%s/%s/messages" % (self.org, self.app))
        if not isinstance(to_ids, list):
            to_ids = [to_ids]

        data = {"target_type": "users",
                "target": to_ids,
                "msg": {"type": "img",
                        "url": img_url,
                        "filename": filename,
                        "secret": str(time() * 1000)},
                "from": from_user_id or self.PUSH_USER_ID,
                "ext": ext if isinstance(ext, dict) else json.loads(ext)}

        headers = yield self.get_app_client_headers()
        response = yield get_async_client().fetch(url,
                                                  method='POST',
                                                  body=json.dumps(data),
                                                  headers=headers)
        if response.code == 200:
            raise gen.Return(response.body)

    @gen.coroutine
    def send_audio_msg(self, to_ids, from_user_id=None, img_url=None, filename=None, length=None, ext={}):
        url = self.EASEMOB_HOST + ("/%s/%s/messages" % (self.org, self.app))
        if not isinstance(to_ids, list):
            to_ids = [to_ids]

        data = {"target_type": "users",
                "target": to_ids,
                "msg": {"type": "audio",
                        "url": img_url,
                        "filename": filename,
                        "length": length,
                        "secret": str(time() * 1000)},
                "from": from_user_id or self.PUSH_USER_ID,
                "ext": ext if isinstance(ext, dict) else json.loads(ext)}

        headers = yield self.get_app_client_headers()
        response = yield get_async_client().fetch(url,
                                                  method='POST',
                                                  body=json.dumps(data),
                                                  headers=headers)
        if response.code == 200:
            raise gen.Return(response.body)

    @gen.coroutine
    def send_video_msg(self, to_ids, from_user_id=None, img_url=None, filename=None, length=None, thumb=None, ext={}):
        url = self.EASEMOB_HOST + ("/%s/%s/messages" % (self.org, self.app))
        if not isinstance(to_ids, list):
            to_ids = [to_ids]

        data = {"target_type": "users",
                "target": to_ids,
                "msg": {"type": "video",
                        "url": img_url,
                        "filename": filename,
                        "length": length,
                        "file_length": 1024 * 1024,
                        "secret": str(time() * 1000),
                        "thumb": thumb,
                        "thumb_secret": str(time() * 1000)},
                "from": from_user_id or self.PUSH_USER_ID,
                "ext": ext if isinstance(ext, dict) else json.loads(ext)}

        headers = yield self.get_app_client_headers()
        response = yield get_async_client().fetch(url,
                                                  method='POST',
                                                  body=json.dumps(data),
                                                  headers=headers)
        if response.code == 200:
            raise gen.Return(response.body)

    @gen.coroutine
    def send_group_msg(self, msg, to_ids, from_user_id=None, ext={}):
        url = self.EASEMOB_HOST + ("/%s/%s/messages" % (self.org, self.app))
        if not isinstance(to_ids, list):
            to_ids = [to_ids]

        data = {"target_type": "chatgroups",
                "target": to_ids,
                "msg": {"type": "txt",
                        "msg": msg},
                "from": from_user_id or self.PUSH_USER_ID,
                "ext": ext if isinstance(ext, dict) else json.loads(ext)}

        headers = yield self.get_app_client_headers()
        response = yield get_async_client().fetch(url,
                                                  method='POST',
                                                  body=json.dumps(data),
                                                  headers=headers)
        if response.code == 200:
            raise gen.Return(response.body)


    @gen.coroutine
    def block_users(self, from_user_id, to_ids):

        url = self.EASEMOB_HOST + ("/%s/%s/users/%s/blocks/users" % (self.org, self.app, from_user_id))

        if not isinstance(to_ids, list):
            to_ids = [to_ids]

        data = {"usernames": to_ids}

        headers = yield self.get_app_client_headers()
        try:
            response = yield get_async_client().fetch(url,
                                                      method='POST',
                                                      body=json.dumps(data),
                                                      headers=headers)
            if response.code == 200:
                raise gen.Return(response.body)
        except HTTPError as e:
            print(e)
            raise gen.Return(False)

    @gen.coroutine
    def unblock_user(self, from_user_id, to_user_id):
        url = self.EASEMOB_HOST + ("/%s/%s/users/%s/blocks/users/%s" % (self.org, self.app, from_user_id, to_user_id))
        headers = yield self.get_app_client_headers()
        response = yield get_async_client().fetch(url,
                                                  method='DELETE',
                                                  headers=headers)
        if response.code == 200:
            raise gen.Return(response.body)


    @gen.coroutine
    def create_chat_group(self, kwargs):
        """创建群， 如果创建者没有在环信注册，则先注册用户"""
        url = self.EASEMOB_HOST + ("/%s/%s/chatgroups" % (self.org, self.app))
        headers = yield self.get_app_client_headers()

        try:
            response = yield get_async_client().fetch(url,
                                                      method='POST',
                                                      headers=headers,
                                                      body=json.dumps(kwargs))
            if response.code == 200:
                raise gen.Return(response.body)
        except HTTPError as e:
            print(e.code, e.response.body)



    @gen.coroutine
    def modify_chat_group(self, gid, kwargs):
        """修改群资料"""
        url = self.EASEMOB_HOST + ("/%s/%s/chatgroups/%d" % (self.org, self.app, gid))
        headers = yield self.get_app_client_headers()
        response = yield get_async_client().fetch(url,
                                                  method='PUT',
                                                  headers=headers,
                                                  body=json.dumps(kwargs))
        if response.code == 200:
            raise gen.Return(response.body)

    @gen.coroutine
    def delete_chat_group(self, gid):
        """解散群"""
        url = self.EASEMOB_HOST + ("/%s/%s/chatgroups/%s" % (self.org, self.app, gid))
        headers = yield self.get_app_client_headers()
        response = yield get_async_client().fetch(url,
                                                  method='DELETE',
                                                  headers=headers)
        if response.code == 200:
            raise gen.Return(response.body)

    @gen.coroutine
    def group_add_member(self, gid, user_id):
        """群添加成员"""
        url = self.EASEMOB_HOST + ("/%s/%s/chatgroups/%d/users/%s" % (self.org, self.app, gid, user_id))
        headers = yield self.get_app_client_headers()
        response = yield get_async_client().fetch(url,
                                                  method='POST',
                                                  headers=headers,
                                                  body="")

        if response.code == 200:
            raise gen.Return(response.body)
        else:
        #     yield self.register_new_user(user_id, md5(user_id + generate_access_token(user_id)).hexdigest())
            raise gen.Return(None)

    @gen.coroutine
    def group_delete_member(self, gid, user_id):
        """群减少成员"""

        url = self.EASEMOB_HOST + ("/%s/%s/chatgroups/%d/users/%s" % (self.org, self.app, gid, user_id))
        headers = yield self.get_app_client_headers()
        response = yield get_async_client().fetch(url,
                                                  method='DELETE',
                                                  headers=headers)
        if response.code == 200:
            raise gen.Return(response.body)


    @gen.coroutine
    def get_chatmessages(self, arg):
        url = self.EASEMOB_HOST + ("/%s/%s/chatmessages?%s" % (self.org, self.app, arg))
        try:
            headers = yield self.get_app_client_headers()
            response = yield get_async_client().fetch(url,
                                                      method='GET',
                                                      headers=headers)
            raise gen.Return(response.body)
        except HTTPError as e:
            raise gen.Return("faild")

    @gen.coroutine
    def check_online(self, user_id):

        url = self.EASEMOB_HOST + ("/%s/%s/users/%s/status" % (self.org, self.app, user_id))
        try:
            headers = yield self.get_app_client_headers()
            response = yield get_async_client().fetch(url,
                                                      method='GET',
                                                      headers=headers)
            raise gen.Return(json.loads(response.body)["data"][str(user_id)] == "online")
        except HTTPError as e:
            raise gen.Return(None)


    def d(self):
        import requests
        payload = {'grant_type': 'client_credentials', 'client_id': self.client_id, 'client_secret': self.client_secret}
        hu = self.EASEMOB_HOST + ("/%s/%s/token" % ("lcylln", "yyq"))
        h = requests.get(hu, payload)
        token = json.loads(h.content)["access_token"]
        headers = {'Authorization': 'Bearer ' + token,
                   'content-type': 'application/json'}
        url = self.EASEMOB_HOST + ("/%s/%s/users" % (self.org, self.app))
        s = requests.get(url, data=json.dumps({"username": "54887015", "password": "b733853dcefc0ad69f02e057608b3e3b"}), headers=headers)
        print(s)


# ss = EMChatAsync()
# ss.d()



# coding=utf-8
import sys
import json, logging
import tornado
import tornado.ioloop
from .libs.singleton import BeanstalkCon
from .libs.beanstalk_route import BsRoute
from traceback import format_exc


class BeanstalkWorker(object):
    def __init__(self, **kwargs):
        self.ROOT = kwargs["ROOT"]
        self.beanstalk_config = kwargs["beanstalk_config"]
        self.workers_path = self.beanstalk_config.get("workers_dir", ["works/beanstalk"])
        self.ioloop = tornado.ioloop.IOLoop.instance()
        self.workers = BsRoute.get_workers(self.ROOT, self.workers_path)

        # 监听的tube的优先级：命令行参数>配置文件指定>消费者指定的所有tube（默认）
        if self.beanstalk_config.get("watch", []):
            self.watch = [watch for watch in self.beanstalk_config["watch"] if watch]
        else:
            self.watch = [event.tube for event in self.workers.values() if hasattr(event, "tube")]
        self.beanstalk_config["watch"] = self.watch
        self.beanstalk = BeanstalkCon.get_connection(**self.beanstalk_config)

    def watching(self):
        for i in self.watch: self.beanstalk.watch(i)
        logging.warning("current watching tubes: %s" % json.dumps(self.beanstalk.watching()))
        logging.warning("Beanstalk Client Running")


    def loop(self):
        job = self.beanstalk.reserve()
        try:
            body = json.loads(job.body)
        except Exception as e:
            logging.warning(format_exc())
            logging.error("event fail job: id=%s, body=%s" % (job.id, job.body))
        else:
            event = body.pop("__event__", None)
            classname, func = event.split(".")
            if classname not in self.workers:
                logging.error("event name %s not found!" % event)
                self.beanstalk.delete(job)
            else:
                try:
                    getattr(self.workers[classname], func)(body)
                except Exception as e:
                    logging.warning(format_exc())
                    logging.error("event fail job: id=%s, event=%s, body=%s" % (job.id, event, job.body))
                else:
                    logging.debug("success finish job: id=%s, event=%s, body=%s" % (job.id, event, job.body))
        finally:
            self.beanstalk.delete(job)

        self.ioloop.add_callback(self.loop)

    def start(self):
        self.watching()
        self.loop()
        self.ioloop.start()


if __name__ == "__main__":
    import logging
    logging.basicConfig(level=logging.DEBUG)
    kwargs = {
        "ROOT": '/Users/lcylln/www/lcyframepy3/lcyframe3-demo',
        "beanstalk_config": {
            "host": "127.0.0.1",
            "port": 11300,
            "workers_dir": "/Users/lcylln/www/lcyframepy3/lcyframe3-demo/works/beanstalk"
        }}
    wk = BeanstalkWorker(**kwargs)
    wk.start()



#!/usr/bin/env python
# -*- coding:utf-8 -*-
import logging
import functools
import sys, os
from .context_start import AutoStartContext
from lcyframe.libs import utils


class ServicesFindet(object):
    """
    Global ContextManage Register module
    """

    def __init__(self, app=None, model=None):
        self.app = app or type("app", (object,), {})
        self.model = model or type("model", (object,), {})

    def __call__(self, c):
        for f in dir(self):
            if f.startswith("import_"):
                r = getattr(self, f)(c)
                if r:
                    logging.debug("conntcion to %s success." % f.split("_")[-1])

    def import_mongo(self, config):
        f = config.get("mongo_config", {})
        if f:
            AutoStartContext.start_mongodb(f)
            from .singleton import MongoCon
            from .permissions import Permission
            self.__mount_attribute("mongo", MongoCon.get_database(**f))

            Permission.mongo = MongoCon.get_database(**f)
            self.__mount_attribute("Permission", Permission.initial())
            return True

    def import_mysql(self, config):
        f = config.get("mysql_config", {})
        if f:
            from .singleton import PyMysqlCon
            self.__mount_attribute("mysql", PyMysqlCon.get_connection(**f))
            return True

    def import_redis(self, config):
        f = config.get("redis_config")
        if f:
            AutoStartContext.start_redis(f)
            from .singleton import RedisCon
            self.__mount_attribute("redis", RedisCon.get_connection(**f))
            return True

    def import_ssdb(self, config):
        f = config.get("ssdb_config")
        if f:
            AutoStartContext.start_ssdb(f)
            from .singleton import SSDBCon
            self.__mount_attribute("ssdb", SSDBCon.get_connection(**f))
            return True

    def import_mq(self, config):
        f = config.get("mq_config")
        if f:
            from .singleton import MQCon
            self.__mount_attribute("mq", MQCon.get_connection(**f))
            return True

    def import_nsq(self, config):
        f = config.get("nsq_config")
        if f:
            from .singleton import NsqCon
            nsq = NsqCon.get_connection(**f)
            self.__mount_attribute("nsq", nsq)

            p = utils.fix_path(os.path.join(config["ROOT"], "producer/nsq"))
            if os.path.exists(p) and not f.get("producer_dir"):
                config["nsq_config"]["producer_dir"] = ["producer/nsq"]

            if f.get("producer_dir"):
                self._import_nsq_producer(nsq, config["ROOT"], config["nsq_config"]["producer_dir"])

            return nsq

    def import_mqtt(self, config):
        f = config.get("mqtt_config")
        if f and f.get("run", True):
            from .singleton import MqttCon
            mqtt = MqttCon.get_connection(**f)
            self.__mount_attribute("mqtt", mqtt)

            p = utils.fix_path(os.path.join(config["ROOT"], "producer/mqtt"))
            if os.path.exists(p) and not f.get("producer_dir"):
                config["mqtt_config"]["producer_dir"] = ["producer/mqtt"]

            if f.get("producer_dir"):
                self._import_mqtt_producer(mqtt, config["ROOT"], config["mqtt_config"]["producer_dir"])

            return mqtt

    def import_celery(self, config):
        f = config.get("celery_config")
        if f and f.get("run", True):
            from .singleton import CeleryCon
            from lcyframe.libs.celery_route import Events

            celeryconfig = f["config_dir"]
            if not celeryconfig.endswith(".py"):
                celeryconfig = celeryconfig.replace(".", "/")
                celeryconfig += ".py"
            if not os.path.exists(utils.fix_path(os.path.join(config["ROOT"], celeryconfig))):
                raise Exception("file does not exist：%s" % celeryconfig)
            celeryconfig = celeryconfig.rstrip(".py")
            celeryconfig = celeryconfig.lstrip("/")
            celeryconfig = celeryconfig.replace("/", ".")
            os.environ['CELERY_CONFIG_MODULE'] = celeryconfig

            p = utils.fix_path(os.path.join(config["ROOT"], "works/celery"))
            if os.path.exists(p) and not f.get("workers_dir"):
                config["celery_config"]["workers_dir"] = ["works/celery"]

            app = CeleryCon.get_connection(**f)
            self._import_celery_worker(app, config["ROOT"], config["celery_config"]["workers_dir"], Events)
            self.__mount_attribute("celery", app)
            return app

    def import_beanstalk(self, config):
        f = config.get("beanstalk_config")
        if not f:
            return
        from .singleton import BeanstalkCon
        from lcyframe.libs.beanstalk_route import Events

        p = utils.fix_path(os.path.join(config["ROOT"], "works/beanstalk"))
        if os.path.exists(p) and not f.get("workers_dir"):
            f["workers_dir"] = ["works/beanstalk"]

        app = BeanstalkCon.get_connection(**f)
        setattr(Events, "beanstalk", app)
        self._import_beanstalk_worker(app, config["ROOT"], f["workers_dir"], Events)
        self.__mount_attribute("beanstalk", app)
        return app

    def import_websocket(self, config):
        f = config.get("websocket_config")
        if not f:
            return
        from .singleton import WebScoketCon
        from lcyframe.libs.websocket_route import Events

        p = utils.fix_path(os.path.join(config["ROOT"], f.get("workers_dir", "works/websocket")))
        if os.path.exists(p) and not f.get("workers_dir"):
            f["workers_dir"] = ["works/websocket"]

        app = WebScoketCon.get_instance(**f)
        setattr(Events, "websocket", app)   # Events.websocket
        self._import_websocket_worker(app, config["ROOT"], f["workers_dir"], Events)
        self.__mount_attribute("websocket", app)
        return app

    def import_mob(self, config):
        f = config.get("sms_config")
        if f:
            from .singleton import SMSCon
            self.__mount_attribute("sms", SMSCon.get_connection(**f))
            return True

    def import_smtp(self, config):
        f = config.get("smtp_config")
        if f:
            from .singleton import SmtpCon
            self.__mount_attribute("smtp", SmtpCon.get_connection(**f))
            return True

    def import_qiniu(self, config):
        f = config.get("qiniu_config")
        if f:
            from .singleton import QinNiuCon
            self.__mount_attribute("qiniu", QinNiuCon.get_connection(**f))
            return True

    def import_emchat(self, config):
        f = config.get("emchat_config")
        if f:
            from .singleton import EmchatCon
            self.__mount_attribute("emchat", EmchatCon.get_connection(**f))
            return True

    def import_token(self, config):
        tc = config["token_config"]
        if "secret" not in tc:
            raise Exception("you must give token secret in you setting.yml")
        self.__mount_attribute("token_config", tc)
        self.__mount_attribute("Token", self._get_jwt(tc))
        return True

    def import_aes(self, config):
        tc = config.get("aes_config", {})
        from .aes import AES
        self.__mount_attribute("aes", AES(tc.get("secret")))
        return True

    def import_graylog(self, config):
        c = config.get("graylog_config")
        if c:
            from .singleton import GrayLogCon
            self.__mount_attribute("graylog", GrayLogCon(**c))
            return True

    def import_errors(self, config):
        return
        model_name = self.fix_module_path(config["errors_dir"]) + "." + "errors.py".rstrip(".py")
        objs = __import__(model_name, fromlist=["*"])
        ApiError = type("ApiError", (object,), {})
        import inspect
        for p in dir(objs):
            if p.startswith("__"):
                continue
            if not inspect.isclass(getattr(objs, p)):
                continue
            if not issubclass(getattr(objs, p), BaseException):
                continue

            setattr(ApiError, p, getattr(objs, p))
        self.__mount_attribute("ApiError", ApiError)

    def __mount_attribute(self, model_name, value):
        if self.app:
            setattr(self.app, model_name, value)
        if self.model:
            setattr(self.model, model_name, value)

    def _get_jwt(self, config):
        """

        :param p:
        :return:
        """

        return functools.partial(self._token, config["secret"], config["expire"])

    def _token(self, secret, expire, uid):
        from .JWT import JwtToken
        return JwtToken(uid, secret, int(expire))

    def _import_nsq_producer(self, app, root_path, produce_path):
        """
        register nsq producer
        :return:
        """
        # for dir in config["nsq_config"]["producer_dir"]:
        #     root_path = os.path.join(config["ROOT"], dir)
        #     new_path = os.path.join(root_path, "__init__.py")
        #     if os.path.exists(new_path):
        #         os.remove(new_path)
        #     if os.path.exists(os.path.join(root_path, "__init__.pyc")):
        #         os.remove(os.path.join(root_path, "__init__.pyc"))
        #
        #     with open(new_path, 'w+', encoding='utf-8') as f:
        #         f.write(pytemplate.get_nsq_producer_init(root_path, dir))
        #
        #     sys.path.append(root_path)
        #     m = ".".join(root_path.split("/")[-2:])
        #     import_module(m)

        for work in produce_path:
            for root, dirs, files in os.walk(os.path.join(root_path, work)):
                for file in files:
                    if file.startswith("__"):
                        continue

                    if file.endswith(".pyc"):
                        continue

                    if not file.endswith(".py"):
                        continue
                    model_name = self.fix_module_path(root.replace(root_path, "")) + "." + file.rstrip(".py")
                    objs = __import__(model_name, fromlist=["*"])

                    for p in dir(objs):
                        if p.startswith("__"):
                            continue

                        # if not p.lower().startswith("publish"):
                        # logging.debug("nsq producer [%s] load fail. it must been start with 'publish'" % p)
                        #    continue

                        if not hasattr(getattr(objs, p), "topic"):
                            continue

                        logging.debug("nsq producer [%s] register success!" % p)
                        setattr(app, p, getattr(objs, p)())

    def _import_mqtt_producer(self, app, root_path, produce_path):
        """
        register mqtt producer
        :return:
        """
        # for dir in config["mqtt_config"]["producer_dir"]:
        #     root_path = os.path.join(config["ROOT"], dir)
        #     new_path = os.path.join(root_path, "__init__.py")
        #     if os.path.exists(new_path):
        #         os.remove(new_path)
        #     if os.path.exists(os.path.join(root_path, "__init__.pyc")):
        #         os.remove(os.path.join(root_path, "__init__.pyc"))
        #
        #     with open(new_path, 'w+', encoding='utf-8') as f:
        #         f.write(yaml2py.get_mqtt_producer_init(root_path, dir))
        #
        #     sys.path.append(root_path)
        #     m = ".".join(root_path.split("/")[-2:])
        #     import_module(m)

        for work in produce_path:
            for root, dirs, files in os.walk(utils.fix_path(os.path.join(root_path, work))):
                for file in files:
                    if file.startswith("__"):
                        continue

                    if file.endswith(".pyc"):
                        continue

                    if not file.endswith(".py"):
                        continue
                    model_name = self.fix_module_path(root.replace(root_path, "")) + "." + file.rstrip(".py")
                    objs = __import__(model_name, fromlist=["*"])

                    for p in dir(objs):
                        if p.startswith("__"):
                            continue

                        # if not p.lower().startswith("publish"):
                        # logging.debug("mqtt producer [%s] load fail. it must been start with 'publish'" % p)
                        #    continue

                        if not hasattr(getattr(objs, p), "topic"):
                            continue

                        logging.debug("mqtt producer [%s] register success!" % p)
                        setattr(app, p, getattr(objs, p)())

    def _import_celery_worker(self, app, root_path, workers_dir, Events):
        """
        register celery tasks
        :return:
        """
        # for dir in config["celery_config"]["workers_dir"]:
        #     root_path = os.path.join(config["ROOT"], dir)
        #     new_path = os.path.join(root_path, "__init__.py")
        #     # if os.path.exists(new_path):
        #     #     os.remove(new_path)
        #     if os.path.exists(os.path.join(root_path, "__init__.pyc")):
        #         os.remove(os.path.join(root_path, "__init__.pyc"))
        #
        #     with open(new_path, 'w+', encoding='utf-8') as f:
        #         f.write(yaml2py.get_celery_works_init(root_path, dir))
        #
        #     sys.path.append(root_path)
        #     m = ".".join(root_path.split("/")[-2:])
        #     import_module(m)

        for work in workers_dir:
            for root, dirs, files in os.walk(utils.fix_path(os.path.join(root_path, work))):
                for file in files:
                    if file.startswith("__"):
                        continue

                    if file.endswith(".pyc"):
                        continue

                    if not file.endswith(".py"):
                        continue

                    model_name = self.fix_module_path(root.replace(root_path, "")) + "." + file.rstrip(".py")
                    objs = __import__(model_name, fromlist=["*"])
                    setattr(Events, objs.__name__.split(".")[-1], objs)

                    for p in dir(objs):
                        if p.startswith("__"):
                            continue

                        if type(getattr(objs, p)) == type:
                            if not hasattr(getattr(objs, p), "queue"):
                                continue

                            if hasattr(app, p) and p != 'BaseEvent':
                                raise Exception("exists same attribute：%s" % p)

                        logging.debug("celery task [%s] register success!" % p)
                        setattr(Events, p, getattr(objs, p))

                    setattr(app, Events.__name__, Events)
        return app

    def _import_beanstalk_worker(self, app, root_path, workers_dir, Events):
        """
        register celery tasks
        :return:
        """
        from lcyframe.libs.beanstalk_route import BsRoute
        workers = BsRoute.get_workers(root_path, workers_dir)
        for class_name, objs in workers.items():
            if not hasattr(objs, "tube"):
                continue
            setattr(app, class_name, type(class_name, (Events,), {"tube": objs.tube, "classname": objs.__name__})())
            logging.debug("beanstalk task class [%s] register success!" % class_name)
        app.workers = workers
        return app

    def _import_websocket_worker(self, app, root_path, workers_dir, Events):
        """
        register websocket tasks
        :return:
        """
        from lcyframe.libs.websocket_route import WebSocketRoute, Agent
        workers = WebSocketRoute.get_workers(root_path, workers_dir)
        for func_name, objs in workers.items():
            if not hasattr(objs, "from_route"):
                continue

            if hasattr(Events, func_name):
                raise Exception("websocket works exists same events：%s" % func_name)

            # setattr(Events, func_name, object)

            # setattr(x, func_name, type(func_name, (Events,), {"event_name": func_name})())

            # x = type(func_name, (Events,), {})()
            # setattr(Events, func_name, getattr(type(func_name, (Events,), {})(), func_name))
            # setattr(Events, func_name, getattr(Events(), func_name))
            setattr(Events, func_name, Agent(Events.websocket, func_name))
            logging.debug("websocket task class [%s] register success!" % func_name)

        setattr(app, Events.__name__, Events)
        return app

        workers_dir = [workers_dir] if not isinstance(workers_dir, (list, tuple)) else workers_dir
        for work in workers_dir:
            for root, dirs, files in os.walk(utils.fix_path(os.path.join(root_path, work))):
                for file in files:
                    if file.startswith("__"):
                        continue

                    if file.endswith(".pyc"):
                        continue

                    if not file.endswith(".py"):
                        continue

                    model_name = self.fix_module_path(root.replace(root_path, "")) + "." + file.rstrip(".py")
                    objs = __import__(model_name, fromlist=["*"])
                    for p in dir(objs):
                        if p.startswith("__"):
                            continue

                        if not hasattr(getattr(objs, p), "from_route"):
                            continue

                        if hasattr(Events, p):
                            raise Exception("exists same attribute：%s" % p)

                        logging.debug("websocket task [%s] register success!" % p)

                        setattr(Events, p, getattr(objs, p))

        setattr(app, Events.__name__, Events)
        return app

    def fix_module_path(self, module_path):
        module_path = utils.fix_path(module_path)
        module_path = module_path.lstrip("/").replace("/", ".").replace("//", ".").replace("/", ".")
        return module_path

    def add_service(self, service_name, service_obj):
        """
        external a service into app and model, only
        :param service_name:
        :param service_obj:
        :return:

        :Make your service and register it before starting
        :example::
            s = ServicesFindet

            def import_mq(self, config):
                f = config["mq_config"]
                from .singleton import MQCon
                mq = MQCon.get_connection(**f)
                return mq

            # 1、app_start:
                s.add_service("mq", import_mq(config))

                app = App(**config)
                app.start()

            # 2、mq_start:
                s.add_service("mq", import_mq(config))

                w = MqttWorker(**config)
                s(w, BaseModel)(config)
                w.start()

            # 3、or, anywhere do it
                from lcyframe.base import BaseHandler, BaseModel
                BaseHandler.mq = import_mq(**f)
                BaseModel.mq = import_mq(**f)

            # Now you can use it like this::
                self.mq ...
                self.model.mq ...
        """

        if service_obj is None:
            raise ValueError("An not None object must be returned in your service_obj()")

        if self.app is None:
            from lcyframe.base import BaseHandler
            self.app = BaseHandler
        if self.model is None:
            from lcyframe.base import BaseModel
            self.model = BaseModel

        setattr(self.app, service_name, service_obj)
        setattr(self.model, service_name, service_obj)




#!/usr/bin/env python
# -*- coding:utf-8 -*-
import sys, os
root_dir = os.path.dirname(os.path.dirname(__file__))
sys.path.append(root_dir)
import platform
import logging
from tornado.options import options, define
from lcyframe.libs import yaml2py, utils
from lcyframe.libs.context_start import AutoStartContext
from lcyframe.libs.singleton import MongoCon
from model.schema.igGenerator_schema import IdGeneratorSchema
from model.igGenerator_model import IdGeneratorModel
# from model.schema.admin_schema import AdminSchema

class InitContext(object):
    config = None
    port = config_file = None

    @classmethod
    def get_context(cls, config_name=None):
        if not config_name:
            logging.warning("platform >>>>>>>>>>>:" + platform.node())
            if len(sys.argv) > 1:
                for item in sys.argv[1:]:
                    arg = item.lstrip("-").lstrip("--")
                    if "=" in arg:
                        name, equals, value = arg.partition("=")
                    else:
                        value = arg

                    if value.isdigit():
                        cls.port = int(value)
                        continue
                    if ".yml" in value:
                        cls.config_file = value
                        continue

                if not cls.config_file:
                    raise Exception("please run like this：python app.py --config=example.yml")

                sys.argv = sys.argv[0]
            elif os.environ.get('PROJECT_CONFIG_FILE'):
                cls.config_file = os.environ["PROJECT_CONFIG_FILE"]
                if not os.path.exists(cls.config_file):
                    raise Exception("配置文件不存在")
            else:
                if platform.node() in ["Online"]:
                    cls.config_file = "server_config.yml"
                elif platform.node() in ["app", "a1320a62c9fd"]:
                    cls.config_file = "test.yml"
                elif platform.node() in ["vm"]:
                    cls.config_file = "vm_config.yml"
                elif platform.node() in ["Mac", "lcyMac.local"]:
                    cls.config_file = "localhost.yml"
                else:
                    cls.config_file = "example.yml"
        else:
            cls.config_file = config_name

        logging.warning(cls.config_file)
        if os.path.isabs(cls.config_file):
            cls.config = yaml2py.load_confog(cls.config_file)
        else:
            cls.config = yaml2py.load_confog(os.path.join(os.path.dirname(__file__), cls.config_file))
        if cls.port:
            cls.config["wsgi"]["port"] = int(cls.port)
        cls.config["config_name"] = cls.config_file.split("/")[-1]
        cls.config["ROOT"] = utils.fix_path(os.path.dirname(os.path.dirname(__file__)))
        os.environ.app_config = cls.config
        return cls.config

    @classmethod
    def init_db(cls):
        AutoStartContext.start_mongodb(cls.config["mongo_config"])


        db = MongoCon().get_database(**cls.config["mongo_config"])
        if not db.id_generator.find().count():
            doc = vars(IdGeneratorSchema())
            doc["uid"] = 10000
            db[IdGeneratorSchema.collection].insert(doc)

        if not db.admin.find().count():

            docs = vars(AdminSchema())
            docs["uid"] = 10000
            docs["nick_name"] = "nick_name"
            docs["pass_word"] = utils.gen_salt_pwd("123456", docs["salt"])
            docs["gid"] = 1
            db[AdminSchema.collection].insert(docs)



# -*- coding: utf-8 -*-
import random, time
from lcyframe.libs.websocket_route import WebSocketRoute
from celery.utils.log import get_task_logger

logger = get_task_logger(__name__)

"""
# 例子一：只接受消息
self.websocket.Event.demo1(**{"a": 1})
# 例子二：接受切需要向客户端发送消息，demo2定义时，需设置第一个位置参数，用法类似于celery的bind
self.websocket.Event.demo2(**{"a": 1})

http客户端发送websocket时的参数为：
{"__event__": "demo1/demo2", "a": 1, ...}
"""


@WebSocketRoute()
def demo1(**message):
    """
    不设置第一个位置参数，只用于接受消息
    """
    # do something
    pass

@WebSocketRoute()
def demo2(socket, **message):
    """
    设置第一个位置参数时，服务会把当前的socket对象绑定到第一个参数上
    socket: work对象，拥有与handler中的self一样的能力
    socket.current_conn: 当前的客户端连接
    message：收到的信息
    """
    # 推送给当前消息来源的的客户端；查询反馈场景
    # socket.send(**message)

    # 发送给指定客户端；定点发送场景
    # conn = socket.conns["uid:id"]   # 所有连接被收集，放置在conns内，key=uid:cliend_id
    # socket.send(conn,**message)

    # 推送消息给所有已连接的客户端（除系统和消息来源方)；聊天室场景
    socket.sendall(**message)



# coding=utf-8
import os
import logging
import hmac
import base64
import random
import time
from pathlib import Path
from datetime import timedelta, datetime, date
from string import ascii_letters
import zlib
import hashlib, string
import json
from bson.objectid import ObjectId
import operator
import re
import traceback
try:
    from JWT import JwtToken
    from Rsa import RSA
    from aes import AES
except: pass

_chars = string.printable[:87] + '_' + string.printable[90:95]
to_ObjectId = lambda a: ObjectId(a) if type(a) != ObjectId else a
to_python = lambda s: json.loads(s)
to_json = lambda obj: json.dumps(obj, ensure_ascii=False, sort_keys=True)
fix_path = lambda path: Path(path).as_posix()
traceback = traceback

num_or_alpha = re.compile("^(?!\d+$)[\da-zA-Z_]{5,10}$")  # 仅数字和字母组合，不允许纯数字,长度5~10
startwith_alpha = re.compile("^[a-zA-Z]{5,10}")  # 仅允许以字母开头,长度5~10
lllegal_char = re.compile('^[_a-zA-Z0-9\u4e00-\u9fa5]+$')  # 仅允许中文、英文、数字,_下划线。但不允许非法字符
email_re = re.compile("^.+\\@(\\[?)[a-zA-Z0-9\\-\\.]+\\.([a-zA-Z]{2,3}|[0-9]{1,3})(\\]?)$")  # 是否是邮箱
phone_re = re.compile("^(0|86|17951)?(13|14|15|16|17|18)[0-9]{9}$")  # 11位手机号
password_normal = re.compile("^(?:(?=.*)(?=.*[a-z])(?=.*[0-9])).{6,12}$")  # 一般密码 密码必须包含字母，数字,任意字符，长度6~12
password_strong = re.compile("^(?:(?=.*[A-Z])(?=.*[a-z])(?=.*[0-9])).{6,12}$")  # 强密码 密码必须包含大小写，数字,任意字符，长度6~12


def random_int(length=6):
    """生成随机的int 长度为length"""
    return ("%0" + str(length) + "d") % random.randint(int("1" + "0" * (length - 1)), int("9" * length))


def gen_random_str(length=6, chars=_chars):
    return ''.join(random.choice(chars) for i in range(length))


def gen_random_sint(length=6):
    """
    获取字符串加数字的随机值
    :param length:
    :return:
    """
    return "".join(random.choice(string.hexdigits) for i in range(length))


def random_string(length=6):
    """生成随机字符串"""
    return ''.join(random.choice(ascii_letters) for i in range(length))


def gen_hmac_key():
    """随机生成长度32位密文"""
    s = str(ObjectId())
    k = gen_random_str()
    key = hmac.HMAC(k, s).hexdigest()
    return key


def enbase64(s):
    """
    编码
    :param s:
    :return:
    """
    if type(s) == bytes:
        return base64.b64encode(s)
    else:
        s = s.encode('utf-8')
        return base64.b64encode(s).decode('utf-8')


def debase64(s):
    """
    解码
    :param s:
    :return:
    """
    bytes_types = (bytes, bytearray)
    return base64.b64decode(s) if isinstance(s, bytes_types) else base64.b64decode(s).decode()


def check_params_date_value(datestr):
    if " " in datestr:
        ymd, hms = datestr.split(" ")
    else:
        ymd, hms = datestr, ""

    seqstr1 = ""
    seqstr2 = ""
    for i in ["-", "/", " ", ":"]:
        if i in ymd:
            seqstr1 = i
        if i in hms:
            seqstr2 = i
    ymd_fmt = []
    hms_fmt = []
    for index, i in enumerate(ymd.split(seqstr1)):
        ymd_fmt.append(["%Y", "%m", "%d"][index])
    for index, i in enumerate(hms.split(seqstr2)):
        hms_fmt.append(["%H", "%M", "%S"][index])
    fmt = ""
    if ymd_fmt:
        fmt += seqstr1.join(ymd_fmt)
    if hms_fmt:
        fmt += " " + seqstr2.join(hms_fmt)
    return string2datetime(datestr, fmt)


class TypeConvert(object):
    """类型转换类, 处理参数"""
    MAP = {int: int,
           float: float,
           bool: bool,
           str: str}

    STR2TYPE = {"int": int,
                "integer": int,
                "string": str,
                "str": str,
                "bool": bool,
                "float": float,
                "list": list,
                "dict": dict,
                "json": json.loads,
                "datetime": check_params_date_value}

    @classmethod
    def apply(cls, obj, raw):
        try:
            tp = type(obj)
            if tp in TypeConvert.MAP:
                return TypeConvert.MAP[tp](raw)
            return obj(raw)
        except Exception as e:
            logging.error("in TypeConvert.apply %s, obj: %s, raw: %s" % (e, obj, raw))
            return None

    @classmethod
    def convert_params(cls, _type, value):
        if _type in ["int", "integer"] and not value:
            value = 0
        try:
            tp = TypeConvert.STR2TYPE[_type]
            return tp(value)
        except Exception as e:
            raise e


def calculate_age(ts):
    """计算年龄"""
    if ts == -1:
        return -1
    born = datetime.fromtimestamp(ts)
    today = date.today()
    return today.year - born.year - ((today.month, today.day) < (born.month, born.day))


def now():
    return int(time.time())


def oid_to_date(oid):
    return int_to_date_string(int(str(oid)[:8], 16))


def int_to_date_string(ts, fm=False):
    fm = fm if fm else "%Y-%m-%d %H:%M"
    try:
        if not ts:
            ts = 0
        return datetime.fromtimestamp(ts).strftime(fm)
    except:
        return datetime.fromtimestamp(time.time()).strftime(fm)


def str2timestamp(date_string, fm=False):
    fm = fm if fm else "%Y-%m-%d"
    return int(time.mktime(time.strptime(date_string, fm)))
    # return int(time.mktime(datetime.strptime(date_string, format).timetuple()))


def timestamp2str(ts, fm=False):
    return int_to_date_string(ts, fm)


def datetime2string(date, fmt="%Y-%m-%d %H:%M:%S"):
    """
     datetime(2020,12,12 12:12:12) ==> 2020-12-12 12:12:12
    """
    return date.strftime(fmt)


def string2datetime(datestr, fmt="%Y-%m-%d %H:%M:%S"):
    """
    2020-12-12 12:12:12 ==> datetime(2020,12,12 12:12:12)
    """
    return datetime.strptime(datestr, fmt)


def get_yesterday_midnight():
    # 获取昨天的午夜时间戳
    return get_today_midnight() - 86400


def get_today_midnight():
    # 获取今天的午夜时间戳
    now = int(time.time())
    return now - now % 86400 - 3600 * 8 - 86400


def get_today_lasttime():
    # 获取今天最后一秒时间戳
    now = int(time.time())
    return now - now % 86400 - 3600 * 8 + 24 * 3600 - 1


def get_delta_day(day=0, is_zoo=False, return_fmt="date"):
    """
    获取-n~n天的时间
    datetime.datetime.fromordinal(datetime.datetime.today().toordinal()) + datetime.timedelta(days=-1)
    :param day:
    :param is_zoo: 是否需要0点时间
    :return:

    s1 = get_delta_day(day=1)
    s2 = get_delta_day(return_fmt=int)
    s3 = get_delta_day(return_fmt="string")
    s4 = get_delta_day(return_fmt=datetime)
    s5 = get_delta_day(return_fmt="%Y-%m-%d")
    s6 = get_delta_day(is_zoo=True)
    """
    now = datetime.now()

    if is_zoo:
        now_day = datetime.fromordinal(datetime.today().toordinal())
    else:
        now_day = datetime.now()

    if day == 0:
        delta_day = now_day
    else:
        # +-n天后
        delta_day = (now + timedelta(days=int(day)))

    # 返回格式
    if return_fmt == int:
        return int(time.mktime(delta_day.timetuple()))
    elif return_fmt in ["date", datetime]:
        return delta_day
    elif return_fmt in ["string"]:
        return delta_day.strftime("%Y-%m-%d %H:%M:%S")
    else:
        return delta_day.strftime(return_fmt)


def get_ts_from_object(s):
    if len(s) == 24:
        return int(s[:8], 16)
    return 0


def compress_obj(dict_obj, compress=True):
    """反序列化dict对象"""
    dict_obj = {"$_key_$": dict_obj} if not isinstance(dict_obj, dict) else dict_obj
    if compress:
        return zlib.compress(to_json(dict_obj))
    return to_json(dict_obj)


def uncompress_obj(binary_string, compress=True):
    """反序列化dict对象"""
    if compress:
        dict_obj = to_python(zlib.decompress(binary_string))
    else:
        dict_obj = to_python(binary_string)

    if "$_key_$" in dict_obj:
        return dict_obj["$_key_$"]
    else:
        return dict_obj


def get_mod(uid, mod=10):
    return int(uid) % mod


def gen_salt(len=6):
    return ''.join(random.sample(string.ascii_letters + string.digits, len))


def gen_salt_pwd(salt, pwd):
    return hashlib.md5((str(salt) + str(pwd)).encode("utf-8")).hexdigest()


def md5(s):
    s = s if isinstance(s, bytes) else str(s).encode("utf-8")
    return hashlib.md5(s).hexdigest()


def pparams(request, params=None):
    print_params = {}
    if params == None:
        params = request.arguments
        params.update(request.body_arguments)
        if hasattr(request, "body_json_arguments"):
            params.update(request.body_json_arguments)

    for k, v in params.items():
        if k not in request.files:
            print_params[k] = v
        else:
            try:
                if "body" in v:
                    tmp = {}
                    tmp.update(v)
                    tmp["body"] = "%d bytes" % len(v["body"])
                    print_params[k] = tmp
                else:
                    print_params[k] = v
            except Exception as e:
                print_params[k] = {}

    return print_params


def version_cmp(version1, version2):
    """比较系统版本号
    v1 > v2 1
    v1 = v2 0
    v1 < v2 -1
    v1: 用户使用的版本
    v2：最新上线的版本
    """

    def normalize(v):
        return [int(x) for x in re.sub(r'(\.0+)*$', '', v).split(".")]

    return operator.gt(normalize(version2), normalize(version1))


def _find_option_with_arg(argv, short_opts=None, long_opts=None):
    """Search argv for options specifying short and longopt alternatives.

    Returns:
        str: value for option found
    Raises:
        KeyError: if option not found.

    Example：
        config_name = _find_option_with_arg(short_opts="-F", long_opts="--config")
    """
    for i, arg in enumerate(argv):
        if arg.startswith('-'):
            if long_opts and arg.startswith('--'):
                name, sep, val = arg.partition('=')
                if name in long_opts:
                    return val if sep else argv[i + 1]
            if short_opts and arg in short_opts:
                return argv[i + 1]
    raise KeyError('|'.join(short_opts or [] + long_opts or []))


def check2json(data):
    if isinstance(data, (list, tuple)):
        for index, item in enumerate(data):
            data[index] = check2json(item)
        return data
    elif isinstance(data, dict):
        for key, value in data.items():
            data[key] = check2json(value)
        return data
    elif isinstance(data, ObjectId):
        return str(data)
    elif isinstance(data, datetime):
        return data.strftime("%Y-%m-%d %H:%M:%S")
    elif data == None:
        return ""
    else:
        return data


def verify_username(username, min: int = 4, max: int = 10):
    """
    校验用户名
    :param username: 待验证用户名
    :param min: 最小长度
    :param max: 最大长度
    :return:
    """
    re_str1 = re.compile("^[a-zA-Z](?!\d+$)[\da-zA-Z_]")  # 非数字开头
    re_str2 = re.compile("^(?!\d+$)[\da-zA-Z_]{%d,%d}$" % (min, max))  # 非数字开头，字母与数字组合，长度4~10

    return re.search(re_str1, username) and re.search(re_str2, username)


def verify_nickname(nickname, min: int = 4, max: int = 10):
    """
    校验昵称
    :param nickname: 待验证昵称
    :param min: 最小长度
    :param max: 最大长度
    :return:
    """
    # 允许中文、字母、数字组合，禁止非法字符,长度4~10
    lllegal_char = re.compile('^[_a-zA-Z0-9\u4e00-\u9fa5]{%d,%d}$' % (min, max))
    return re.search(lllegal_char, nickname)


def verify_password(password, min: int = 4, max: int = 10, strong_level=1):
    """
    校验密码
    :param password: 待验证密码
    :param min: 最小长度
    :param max: 最大长度
    :param strong: 密码强度等级
            以下所有规则下的密码都禁止包含空格、表情、汉字
            1、一般密码：符合长度且不能是重复或者连续的数字
            2、正常密码：必须包含字母、数字
            3、强密码：必须包含字母（含大写或小写）、数字
            4、强密码：必须包含字母、数字、特殊英文字符
            5、史上最强密码：必须包含字母（含大小写）、数字、特殊英文字符

    :return:
    """
    password_normal = re.compile(
        "^(?:(?=.*)(?=.*[A-Za-z])(?=.*[0-9])).{%d,%d}$" % (min, max))  # 一般密码 密码必须包含字母(大写或小写)，数字，长度6~12
    password_strong = re.compile(
        "^(?:(?=.*[A-Z])(?=.*[a-z])(?=.*[0-9])).{%d,%d}$" % (min, max))  # 强密码 密码必须含字母（含大写），数字，任意字符，长度6~12
    special_char = ["~", "!", "@", "#", "$", "%", "^", "&", "*", "(", ")", "_", "+", "=", "|", ".", ",", "?", "{", "}",
                    "[", "]", ":"]

    if strong_level == 1:
        if len(password) < min or len(password) > max:
            result = False
        elif is_simple(password) == True:
            result = False
        else:
            result = True
    elif strong_level == 2:
        result = re.search(password_normal, password)
    elif strong_level == 3:
        result = re.search(password_strong, password)
    elif strong_level == 4:
        result = re.search(password_normal, password)
        if result:
            result = any(c in special_char for c in password)
    elif strong_level == 5:
        result = re.search(password_strong, password)
        if result:
            result = any(c in special_char for c in password)
    else:
        result = False

    if not result:
        return False

    # 禁止中文、标签
    if extract_chinese(password):
        return False

    # 禁止非法全角字符
    if extract_illegal_char(password):
        return False

    # 是否含表情
    if password != replace_emoji(password):
        return False

    return True


def verify_phone(phone):
    """
    校验手机号
    :param phone: 待验证phone
    :param min: 最小长度
    :param max: 最大长度

    :return:
    """
    phone_re = re.compile("^(0|86|17951)?(13|14|15|16|17|18)[0-9]{9}$")  # 11位手机号
    return re.search(phone_re, phone)


def verify_email(email, min: int = 7, max: int = 50):
    """
    校验邮箱
    :param phone: 待验证邮箱
    :param min: 最小长度
    :param max: 最大长度

    :return:
    """
    # 是否是邮箱,且长度在7~50之间
    email_re = re.compile("^.+\\@(\\[?)[a-zA-Z0-9\\-\\.]+\\.([a-zA-Z]{2,3}|[0-9]{1,3})(\\]?){%d,%d}$" % (min, max))
    return re.search(email_re, email)


def verify_ip(ip, IPv6=True):
    """
    校验IP
    :param ip: 待验证ip
    :param min: 最小长度
    :param max: 最大长度

    :return:
    """
    ipv4_re = r"^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$"  # IPV4
    ipv6_re = r"^(?:[A-F0-9]{1,4}:){7}[A-F0-9]{1,4}$"  # IPV6
    if IPv6:  # 大小写不敏感
        return re.search(ipv4_re, ip, re.I) or re.search(ipv6_re, ip)
    else:
        return re.search(ipv4_re, ip)

def verify_domain(domain, mtype=1):
    """
    校验域名
    mtype:
        1判断是否为合法域名；
        2提取域名有效部分：https://7w.7.wx.baidu.com >> 7w.7.wx.baidu.com
    :return:
    """
    if domain.startswith("https://"):
        protocol = "https://"
    elif domain.startswith("http://"):
        protocol = "http://"
    else:
        protocol = ""
    domain_re = r'\b((?=[a-z0-9-]{1,63}\.)(xn--)?[a-z0-9]+(-[a-z0-9]+)*\.)+[a-z]{2,63}\b'
    result = re.search(domain_re, domain, re.I)
    if result:
        if mtype == 1:
            return result.group() == domain.strip(protocol)
        else:
            return result.group()
    else:
        return False

def extract_domain(domain):
    """
    提取顶级域名
    domain: http://news.baidu.com
    tld = extract_domain(domain)
    >>> ExtractResult(subdomain='news', domain='baidu', suffix='com')
    >>> tld[0]
    >>> tld[1]
    >>> tld[2]
    tld.registered_domain
    >>> baidu.com
    """
    import tldextract
    tld = tldextract.extract(domain)
    return tld

def extract_chinese(str):
    """
    提取中文
    :param str:
    :return:
    """

    regex_str = ".*?([\u4E00-\u9FA5]+).*?"
    match_obj = re.findall(regex_str, str)
    return match_obj


def extract_illegal_char(str):
    """
    提取非法半角字符，空格
    :param str:
    :return:
    """
    regex_str = "([！￥……（）——，。、？；：【】「」《》“”‘'  ]+).*?"
    match_obj = re.findall(regex_str, str)
    return match_obj


def extract_illegal_char2(str):
    """
    性能较差
    判断是否含有特殊非法字符
    :param str:
    :return:
    """
    # 所有非法字符
    all_illegal_char = range(0x0, 0x10ffff)  # 总数110万个
    for s in str:
        if s in all_illegal_char:  # 判断一个字符大约需要20ms
            return False
    return True


def change_emoji(char_str):
    """
    将表情转为字符
    :param str: 123🆚456
    :param result: 123:VS_button:456

    :param str: 表情🏷
    :param result: 表情:label:

    :return:
    """
    import emoji
    return emoji.demojize(char_str)


def replace_emoji(char_str, rep_str=''):
    """
    将字符串中的表情替换为指定的字符，默认替换为空，即过滤表情
    常用语用户名、昵称注册等禁止表情的输入
    :param char_str:
    :param rep_str:
    :return:
    """
    try:
        co = re.compile(u'[\U00010000-\U0010ffff]')
    except re.error:
        co = re.compile(u'[\uD800-\uDBFF][\uDC00-\uDFFF]')
    return co.sub(rep_str, char_str)


def is_simple(password):
    """
    判断字符串密码是否为简单重复或者纯数字或连续的数字
    若是，返回True
    若不是，返回False
    :param password:
    :return:
    """
    if password.isdigit():
        password = [int(n) for n in password]
        if len(set(password)) == 1:
            return True

        for index, a in enumerate(password[: -1]):
            b = password[index + 1]
            if (b - a) != 1:
                return False
        return True
    else:
        password = [n for n in password]
        if len(set(password)) == 1:
            return True
        else:
            return False


def transfer_str(str):
    """
    mongo 正则匹配 转义
    :param str:
    :return:
    """
    new_str = ""
    special = ['/', '^', '$', '*', '+', '?', '.', '(', ')']
    for c in str:
        if c in special:
            new_str += '\\'
        new_str += c
    return new_str


def supper_format(filename, format_list):
    """
    判断文件是否在指定的格式内
    :param filename: abc.e.f.tar.gz
    :param format_list: ["rar", "tar", "tar.gz"]
    :return: True
    """
    return any(filter(lambda x: filename.lower().endswith(x.lower()), format_list))


def get_filename_format(filename, format_list):
    """
    截取文件名和格式名
    针对压缩包名称较为复杂的情况
    :param filename: abc.e.f.tar.gz
    :param format_list: ["rar", "tar", "tar.gz"]
    :return: abc.e.f
    """
    if "." not in filename:
        return filename, ""

    exists_format = False

    for i in format_list:
        if not i.startswith("."):
            suffix = "." + i
        else:
            suffix = i
        if filename.lower().endswith(i):
            name, format = filename.rsplit(suffix, 1)[0], i
            exists_format = True
            break

    if exists_format:
        return name, format
    else:
        return filename.rsplit(".", 1)


def auto_rename(name, uncompress_path, n=1):
    """
    指定一个解压位置，若有同名文件夹存在，则自动重命名
    :param name:
    :param uncompress_path:
    :param n:
    :return:
    """
    while n < 100:
        if n > 1:
            dst_path = os.path.join(uncompress_path, name + "-" + str(n))
        else:
            dst_path = os.path.join(uncompress_path, name)

        if os.path.exists(dst_path):
            n += 1
            return auto_rename(name, uncompress_path, n)
        else:
            if n > 1:
                return name + "-" + str(n), dst_path
            else:
                return name, dst_path
    raise


def undecompress(compress_path, uncompress_path):
    """
    解压包
    :param compress_path: 压缩包目前所在目录: /code/123.zip
    :param uncompress_path: 需要解压至该目录:/code/newname
    :param decompress_format: 支持的解压格式列表["zip", "rar", "7z", "tar", "tbz2", "tgz", "tar.bz2", "tar.gz", "tar.xz", "tar.Z"]
    :return:
    """
    import patoolib
    try:
        if not os.path.exists(uncompress_path): os.makedirs(uncompress_path)
        patoolib.extract_archive(compress_path, outdir=uncompress_path)
    except Exception as e:
        os.remove(uncompress_path)
        logging.error(str(e))
    else:
        return True


def pdf2img(pdf_path, to_path):
    '''
    # 将PDF转化为图片
    pdf_path pdf文件的路径
    to_path 保存文件夹目录
    zoom_x x方向的缩放系数
    zoom_y y方向的缩放系数
    rotation_angle 旋转角度
    '''
    try:
        import fitz
        doc = fitz.open(pdf_path)
        print("PDF转图片，任务文件：%s" % pdf_path)

        ts = now()
        for pg in range(doc.pageCount):
            print("\r共%s页,正在转换第%s/%s张" % (doc.pageCount, pg + 1, doc.pageCount), end="")
            page = doc[pg]
            rotate = int(0)
            # 每个尺寸的缩放系数为8，这将为我们生成分辨率提高64倍的图像。
            zoom_x = 6.0
            zoom_y = 6.0
            trans = fitz.Matrix(zoom_x, zoom_y).prerotate(rotate)
            pm = page.get_pixmap(matrix=trans, alpha=False)
            save_name = '{:01}.png'.format(pg + 1)
            pm.save(os.path.join(to_path, save_name))
        print()
        print("耗时:%s秒" % str(now() - ts))
    except Exception as e:
        print(str(e))

def imagecode(stype=4, args_value=None, return_type=2, filename=None, **kwargs):
    """
    生成图形验证码
    stype: 验证码类型
    args_value： 指定生成参数；值可以是：指定的验证码，生成的长度；加减法符号
    return_type： 返回类型 1 返回字节码；2返回base64；3保存到指定文件
    filename： 保存路径/名称.png
    kwargs：生成参数
    dict(
        font_color_values=font_color_values,
        font_background_value='#87CEFF',
        draw_dots=True,
        dots_width=5,
        dots_count=200,
        draw_lines=True,
        lines_width=4,
        lines_count=10,
        lines_background_value="#ffffff",
    )

    例子：
    print(imagecode(1, "123456", return_type=2))
    print(imagecode(2, 6, return_type=2))
    print(imagecode(3, 6, return_type=3))
    print(imagecode(4, 6, return_type=3))
    print(imagecode(5, "+", return_type=3))

    """
    from .gvcode import VFCode
    kwargs = kwargs if kwargs else dict(
        font_background_value='#87CEFF',
        draw_dots=True,
        dots_width=5,
        dots_count=200,
        draw_lines=True,
        lines_width=4,
        lines_count=10,
        lines_background_value="#ffffff",
    )
    vc = VFCode(**kwargs)

    mp = {
        # 自定义验证码
        # vc.generate('abcd')
        1: vc.generate,

        # 数字验证码（默认5位）
        # vc.generate_digit()
        # vc.generate_digit(4)
        2: vc.generate_digit,

        # 字母验证码（默认5位）
        # vc.generate_alpha()
        # vc.generate_alpha(5)
        3: vc.generate_alpha,

        # 数字字母混合验证码（默认5位）
        # vc.generate_mix()
        # vc.generate_mix(6)
        4: vc.generate_mix,

        # 数字加减验证码（默认加法）
        # 数字加减验证码（加法）
        # vc.generate_op('+')
        # 数字加减验证码（减法）
        # vc.generate_op('-')
        # 数字乘法，字母x
        # vc.generate_op('x')
        5: vc.generate_op,
    }
    if stype not in [1, 2, 3, 4, 5]:
        raise Exception("类型错误")

    if args_value is not None:
        mp[stype](args_value)
    else:
        mp[stype]()

    code = vc.code
    if return_type == 1:    # 返回字节码
        code_data = vc.get_img_bytes()
    elif return_type == 2:
        code, code_data = vc.get_img_base64()
    else:
        vc.save(filename)
        code_data = filename

    return code, code_data




import os
from lcyframe.mq_server import MqWorker
from lcyframe.libs import yaml2py
from base import BaseModel
from lcyframe import ServicesFindet
from context import InitContext

config = InitContext.get_context()

w = MqWorker(**config)
yaml2py.impmodule(BaseModel, model_dir="model")
ServicesFindet(w, BaseModel)(config)
w.start()


from lcyframe import MqttTask, ReadMqtt
from lcyframe import BaseModel

@MqttTask(topic="topic1", qos=2)
class ExampleEvent(ReadMqtt, BaseModel):

    @classmethod
    def on_create_user(cls, **msg):
        print(msg, "mqtt")
        return msg




from tornado.websocket import websocket_connect
from tornado.gen import engine
from tornado.ioloop import IOLoop

"""
基于ioloop连接到已handler的形式启动的tornado.websocket服务
url：ws://192.168.2.50:6677/wechat
类型http的模式提交ws消息

区别：https://www.jianshu.com/p/59b5594ffbb0/
socket:TCP/IP协议的封装，适用服务端与其他服务端
websocket:是html5规范中的一个部分,它借鉴了socket这种思想提供web应用程序和服务端全双工通信应用层协议.适用app、web应用
"""

class WebSocketConn(object):
    @engine
    def connection(self):
        conn = yield websocket_connect("ws://0.0.0.0:12345")
        while True:
            # msg = yield conn.read_message()
            # if msg is None: break
            # print("收到信息：%s" % msg)

            conn.write_message("回复消息：%s" % "回复消息，时代大厦")


if __name__ == "__main__":
    ioloop = IOLoop.instance()
    ws = WebSocketConn()
    ioloop.add_callback(ws.connection)
    ioloop.start()


#!/usr/bin/env python
# -*- coding:utf-8 -*-

from bson.objectid import ObjectId
from base import BaseSchema
import time


class WebSocketSchema(BaseSchema):
    """
    模拟聊天室
    """

    collection = "websocket"

    def __init__(self):
        self._id = ObjectId()




#!/usr/bin/env python
# -*- coding:utf-8 -*-

from lcyframe import route
from lcyframe import funts
from base import BaseHandler
from tornado.websocket import WebSocketHandler

"""
WebSocket协议
已handler的模式启动WebSocket服务，实现模拟聊天室功能
https://www.tornadoweb.org/en/branch5.1/_modules/tornado/websocket.html
"""

@route("/websocket")
class WebSocketHandlerHandler(WebSocketHandler, BaseHandler):
    """
    websocket协议
    仅接受前端ws连接，如app采用多进程部署，每个进程都会有一个websocket服务，且相互隔离；所以不同的请求，可能返回不同的socket连接
    连接地址：url = ws://localhost:8000/wechat?nickname=张三
    .. js连接代码：
    var ws = new WebSocket(url);
      ws.onopen = function() {
         ws.send("Hello, world");
      };
      ws.onmessage = function (evt) {
         alert(evt.data);
      };

    .. python 连接：
    from tornado.websocket import websocket_connect
    from tornado.gen import engine
    from tornado.ioloop import IOLoop
    class WebSocketConn(object):
        @engine
        def connection(self):
            conn = yield websocket_connect("ws://192.168.2.50:6677/wechat?nickname=四张K")
            while True:
                # msg = yield conn.read_message()
                # if msg is None: break
                # print("收到信息：%s" % msg)

                conn.write_message("回复消息：%s" % "回复消息，时代大厦")

    if __name__ == "__main__":
        ioloop = IOLoop.instance()
        ws = WebSocketConn()
        ioloop.add_callback(ws.connection)
        ioloop.start()

    .. 在线测试：http://coolaf.com/tool/chattest
    """

    users = set()                   # 用来存放在线用户的容器
    def open(self):
        print('收到新的WebSocket连接')
        self.users.add(self)        # 建立连接后添加用户到容器中
        nickname = self.get_argument("nickname")
        for u in self.users:        # 向已在线用户发送消息
            u.write_message(u"[%s(%s)] 进入聊天室" % (nickname, self.request.remote_ip))

    def on_message(self, message):
        nickname = self.get_argument("nickname")
        # message = json.loads(message)
        print(type(message),message)
        for u in self.users:        # 向在线用户广播消息
            if u == self:continue
            u.write_message(u"[%s(%s)] 说：%s<br> &nbsp&nbsp&nbsp&nbsp" % (nickname, self.request.remote_ip, message))

    def on_close(self):
        nickname = self.get_argument("nickname")
        print(u"[%s(%s)] 离开聊天室" % (nickname, self.request.remote_ip))
        self.users.remove(self)     # 用户关闭连接后从容器中移除用户

        # 通知其他在线用户
        for u in self.users:
            u.write_message(u"[%s] 离开聊天室" % (nickname))

    def check_origin(self, origin):
        # 允许WebSocket的跨域请求
        return True
            
        




#!/usr/bin/env python
# -*- coding:utf-8 -*-

"""
常数配置
"""
# 全站基本状态码：只可引用，禁止新建
INITIAL = 0     # 初态 默认值
ENABLE = 1      # 中间态 可用的 正常的 开启的 激活的
DISABLE = -1    # 中间态 禁止 关闭 失效

STATE1 = ENABLE # 中间态
STATE2 = 2      # 中间态
STATE3 = 3      # 中间态
STATE4 = 4      # 中间态
STATE5 = 5      # 中间态
STATE6 = 6      # 中间态
STATE7 = 7      # 中间态
STATE8 = 8      # 中间态
STATE9 = 9      # 中间态
STATE10 = 10    # 中间态

# 以下常用语具备事务特性的场景
SUCCESS = OK = 100      # 终态（不可逆） 完成 成功
FAIL = DELETE = -100    # 终态（不可逆）删除

######################### 以下根据业务模块，指定不同的变量，引用基本状态码 ##########################

# 模块1
order_init = INITIAL            # 下单
order_confirm = ENABLE          # 确认
order_payment = STATE2          # 已支付
order_payment_fail = STATE3     # 支付失败
order_payment_success = STATE4  # 支付成功，
order_send_init = STATE5        # 代发货
order_send_enable = STATE6      # 已发货
order_send_success = STATE7     # 已签收
order_success = SUCCESS         # 已确认，交易完结
order_fail = DELETE             # 订单关闭，库存恢复

# 用户
user_register = INITIAL     # 待审批
user_confirm = SUCCESS      # 审批通过
user_fail = DISABLE         # 审批未通过
user_dispost = STATE2       # 已禁言
user_delete = DELETE        # 已删除

# 项目包状态
PROJECT_INIT = INITIAL      # 初始化
PROJECT_SELECT = STATE1     # 查询中
PROJECT_SUCCESS = SUCCESS   # 成功
PROJECT_FAIL = DISABLE      # 失败
PROJECT_DELETE = DELETE     # 已删除

############################## 映射关系 #####################################
# 上传文件保存目录
FILE_TYPE2PATH = {1: "case", 2: "files"}

# 文书类型
WENSHU_S6 = {0: "全部", 1: "判决书", 2: "裁定书", 3: "调解书", 4: "决定书", 5: "通知书", 9: "令", 10: "其他"}
# 案件公开类型
WENSHU_S43 = {1: "文书公开", 2: "信息公开"}

# 债务人类型
CASE_INT2STR = {
    1: "个人", 2: "有限责任公司", 3: "股份有限公司", 4: "有限合伙企业",
    5: "外商独资企业", 6: "个人独资企业", 7: "国有独资公司", 8: "其他"
}
CASE_STR2INT = {v: k for k, v in CASE_INT2STR.items()}

# 法律状态
LAW_INT2STR = {
    1: "未诉讼", 2: "诉讼中", 3: "已获生效判决", 4: "执行中", 5: "执行终结",
    6: "执行终本", 7: "执行中止", 8: "执行中止", 9: "破产清算"
}
LAW_STR2INT = {v: k for k, v in LAW_INT2STR.items()}

# 资产类型
ASSETS_MAPING = {
    100: {"name": "土地", "items": {101: "工业用地", 102: "商业用地", 103: "住宅用地", 104: "商住用地", 105: "农业用地", 106: "其它用地"}},
    200: {"name": "房产", "items": {201: "住宅", 202: "商业", 203: "别墅", 204: "厂房", 205: "办公", 206: "在建工程", 207: "酒店", 208: "车位"}},
    300: {"name": "其他不动产", "items": {301: "林权", 302: "其它不动产"}},
    400: {"name": "动产", "items": {401: "机器设备", 402: "存货", 403: "应收账款", 404: "票据", 405: "存款", 406: "股权", 407: "基金", 408: "车辆", 409: "船舶", 410: "飞机", 411: "碳排放权", 412: "其它动产"}},
}
# 后端使用
ASSETS_INT2STR = {}
for item in ASSETS_MAPING.values():
    ASSETS_INT2STR.update(item["items"])
ASSETS_STR2INT = {v: k for k, v in ASSETS_INT2STR.items()}
# 前端使用
ASSETS_MAPING2 = []
for code, item in ASSETS_MAPING.items():
    ASSETS_MAPING2.append({
        "lable": item["name"],
        "value": code,
        "children": [{"lable": v, "value": k} for k, v in item["items"].items()]
    })

# 担保方式
GUARANTEE_INT2STR = {1: "担保", 2: "抵押", 3: "质押", 4: "保证+抵押", 5: "保证+质押", 6: "抵押+质押", 7: "保证+抵押+质押"}
GUARANTEE_STR2INT = {v: k for k, v in GUARANTEE_INT2STR.items()}
# 保证方式
GUARANTEE2_INT2STR = {1: "一般保证", 2: "连带责任保证"}
GUARANTEE2_STR2INT = {v: k for k, v in GUARANTEE2_INT2STR.items()}

# 建筑结构
HOUSESTRUCT_INT2STR = {1: "混合", 2: "钢混", 3: "砖混", 4: "砖木", 5: "钢", 6: "其他"}
HOUSESTRUCT_STR2INT = {v: k for k, v in HOUSESTRUCT_INT2STR.items()}

# 城市对照表
from .city import city_mapping
CITY_INT2STR = {}           # {110000: '北京市', 110100: '东城区', 110200: '西城区', 110300: '崇文区'....}
CITY_STR2INT = {}           # {"北京市": 110000, "东城区": 110100, "西城区": 110200....}
for item in city_mapping:
    CITY_INT2STR[item["code"]] = item["name"]
    CITY_STR2INT[item["name"]] = item["code"]
    for d in item["city"]:
        CITY_INT2STR[d["code"]] = d["name"]
        CITY_STR2INT[d["name"]] = d["code"]
# 返回前端下拉框
CITY_MAPPING2 = city_mapping

# 0否 1是 -1未知
YESORNO_INT2STR = {0: "否", 1: "是", -1: "未知"}
YESORNO_STR2INT = {v: k for k, v in YESORNO_INT2STR.items()}

# 抵（质）押人与贷款人的关系/与债务人关系
RELATION_INT2STR = {1: "法定代表人", 2: "股东", 3: "关联公司", 4: "其他"}
RELATION_STR2INT = {v: k for k, v in RELATION_INT2STR.items()}

# 诉讼时效
lawsuit_state_mp = {0: "已过", 1: "未过"}
lawsuit_state_mp2 = {"已过": 0, "未过": 1}
# 抵押类型
collateral_type_mp = {1: "抵押", 2: "质押"}
collateral_type_mp2 = {"抵押": 1, "质押": 2}
# 土地性质
land_type_mp = {1: "国有", 2: "集体"}
land_type_mp2 = {"国有": 1, "集体": 2}
# 土地获得方式
land_own_type_mp = {1: "出让", 2: "划拨", -1: "未知"}
land_own_type_mp2 = {"出让": 1, "划拨": 2, "未知": -1}
# 房产出租情况 1空置 2自用 3出租 4其他
house_lease_mp = {1: "出租", 2: "自用", 3: "空置", 4: "其他"}
house_lease_mp2 = {v: k for k, v in house_lease_mp.items()}
# 抵前/抵后出租 0抵前 1抵后 -1未知
house_order_mp = {1: "抵前", 2: "抵后", -1: "未知"}
house_order_mp2 = {v: k for k, v in house_order_mp.items()}
# 保证人类型 1 企业 2 自然人
guarantor_ptype_mp = {1: "企业", 2: "自然人"}
guarantor_ptype_mp2 = {v: k for k, v in guarantor_ptype_mp.items()}

############################################## files #################################################
# 压缩格式
ziprar_format = ["zip", "rar"]
tar_format = ["tar", "tbz2", "tgz", "tar.bz2", "tar.gz", "tar.xz", "tar.Z"]
decompress_format = ziprar_format + tar_format

image_format = ["jpg", "jpeg", "png"]

# 支持上传的格式
support_upload = ["xlsx", "xls",
                  "doc", "docx",
                  "pdf",
                  ] + decompress_format + image_format



#!/usr/bin/env python
# -*- coding:utf-8 -*-

from bson.objectid import ObjectId
from base import BaseSchema
import time


class UploadSchema(BaseSchema):
    """
    上传文件
    """

    collection = "upload"

    def __init__(self):
        self._id = ObjectId()




#!/usr/bin/env python
# -*- coding:utf-8 -*-
from base import BaseModel
from model.schema.websocket_schema import WebSocketSchema

class WebsocketModel(BaseModel, WebSocketSchema):

    @classmethod
    def _parse_data(cls, doc, **kwargs):
        """
        组装、修正单条记录
        该方法若存在，框架会自动使用
        若不存在，则模式使用框架内置方法
        :param doc: 单条数据
        :param kwargs:
        :return:
        """
        if not doc:
            return {}

        if "_id" in doc:
            doc["websocket_id"] = doc.pop("_id", "")

        return doc

    @classmethod
    def get_websocket(cls, websocket_id, **kwargs):
        """
        详情
        :return:
        :rtype:
        """
        return cls.find_one_by_oid(websocket_id)


    @classmethod
    def get_websocket_by_spec(cls, spec, **kwargs):
        """
        详情
        :return:
        :rtype:
        """
        return cls.find_one(spec)


    @classmethod
    def get_websocket_list_by_last_id(cls, last_id, count, **kwargs):
        """
        前端列表
        :return:
        :rtype:
        """
        spec = {}
        spec.update(kwargs.get("spec", {}))
        data_list, last_id = cls.find_list_by_last_id(spec,
                                                      count,
                                                      sort=[("create_at", -1), ],
                                                      fields=kwargs.get("fields", None),
                                                      last_id_field=False)
        return data_list, last_id

    @classmethod
    def get_websocket_list_by_page(cls, page, count, **kwargs):
        """
        后台列表
        :return:
        :rtype:
        """
        spec = {}
        spec.update(kwargs.get("spec", {}))
        data_list, pages, counts = cls.find_list_by_page(spec,
                                                         page,
                                                         count,
                                                         sort=[("create_at", -1), ],
                                                         fields=kwargs.get("fields", None))
        return data_list, pages, counts

    @classmethod
    def create_websocket(cls, *args, **kwargs):
        """
        创建
        :return:
        :rtype:
        """
        pass

    @classmethod
    def modify_websocket(cls, *args, **kwargs):
        """
        修改
        :return:
        :rtype:
        """
        pass

    @classmethod
    def delete_websocket(cls, websocket_id, **kwargs):
        """
        删除
        :return:
        :rtype:
        """
        return cls.update({"_id": websocket_id}, {"state": -1})







# coding=utf-8
import logging

import tornado.ioloop

from .libs import tornado_crontab


class crontabs():
    def __init__(self, **kwargs):
        self.task = kwargs

    def run(self):
        logging.basicConfig(level=logging.DEBUG)
        for k, obj in self.task.items():
            tornado_crontab.CronTabCallback(obj, k).start()
        tornado.ioloop.IOLoop.instance().start()

if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    tornado_crontab.CronTabCallback(lambda a: a, "* * * * *").start()
    tornado.ioloop.IOLoop.instance().start()